from __future__ import annotations

import datetime
from collections.abc import Mapping
from typing import TYPE_CHECKING, Any, TypeVar, cast

from attrs import define as _attrs_define
from attrs import field as _attrs_field
from dateutil.parser import isoparse

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.user_identity_identity_data import UserIdentityIdentityData


T = TypeVar("T", bound="UserIdentity")


@_attrs_define
class UserIdentity:
    """
    Attributes:
        id (str):
        identity_id (str):
        user_id (str):
        identity_data (UserIdentityIdentityData):
        provider (str):
        created_at (datetime.datetime):
        last_sign_in_at (datetime.datetime | None | Unset):
        updated_at (datetime.datetime | None | Unset):
    """

    id: str
    identity_id: str
    user_id: str
    identity_data: UserIdentityIdentityData
    provider: str
    created_at: datetime.datetime
    last_sign_in_at: datetime.datetime | None | Unset = UNSET
    updated_at: datetime.datetime | None | Unset = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        id = self.id

        identity_id = self.identity_id

        user_id = self.user_id

        identity_data = self.identity_data.to_dict()

        provider = self.provider

        created_at = self.created_at.isoformat()

        last_sign_in_at: None | str | Unset
        if isinstance(self.last_sign_in_at, Unset):
            last_sign_in_at = UNSET
        elif isinstance(self.last_sign_in_at, datetime.datetime):
            last_sign_in_at = self.last_sign_in_at.isoformat()
        else:
            last_sign_in_at = self.last_sign_in_at

        updated_at: None | str | Unset
        if isinstance(self.updated_at, Unset):
            updated_at = UNSET
        elif isinstance(self.updated_at, datetime.datetime):
            updated_at = self.updated_at.isoformat()
        else:
            updated_at = self.updated_at

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "id": id,
                "identity_id": identity_id,
                "user_id": user_id,
                "identity_data": identity_data,
                "provider": provider,
                "created_at": created_at,
            }
        )
        if last_sign_in_at is not UNSET:
            field_dict["last_sign_in_at"] = last_sign_in_at
        if updated_at is not UNSET:
            field_dict["updated_at"] = updated_at

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        from ..models.user_identity_identity_data import UserIdentityIdentityData

        d = dict(src_dict)
        id = d.pop("id")

        identity_id = d.pop("identity_id")

        user_id = d.pop("user_id")

        identity_data = UserIdentityIdentityData.from_dict(d.pop("identity_data"))

        provider = d.pop("provider")

        created_at = isoparse(d.pop("created_at"))

        def _parse_last_sign_in_at(data: object) -> datetime.datetime | None | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, str):
                    raise TypeError()
                last_sign_in_at_type_0 = isoparse(data)

                return last_sign_in_at_type_0
            except (TypeError, ValueError, AttributeError, KeyError):
                pass
            return cast(datetime.datetime | None | Unset, data)

        last_sign_in_at = _parse_last_sign_in_at(d.pop("last_sign_in_at", UNSET))

        def _parse_updated_at(data: object) -> datetime.datetime | None | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, str):
                    raise TypeError()
                updated_at_type_0 = isoparse(data)

                return updated_at_type_0
            except (TypeError, ValueError, AttributeError, KeyError):
                pass
            return cast(datetime.datetime | None | Unset, data)

        updated_at = _parse_updated_at(d.pop("updated_at", UNSET))

        user_identity = cls(
            id=id,
            identity_id=identity_id,
            user_id=user_id,
            identity_data=identity_data,
            provider=provider,
            created_at=created_at,
            last_sign_in_at=last_sign_in_at,
            updated_at=updated_at,
        )

        user_identity.additional_properties = d
        return user_identity

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from enum import Enum


class EventResultStatus(str, Enum):
    ERROR = "error"
    PROCESSING = "processing"
    STUCK = "stuck"
    SUCCESS = "success"
    WAITING = "waiting"

    def __str__(self) -> str:
        return str(self.value)



from http import HTTPStatus
from typing import Any
from urllib.parse import quote

import httpx

from ... import errors
from ...client import AuthenticatedClient, Client
from ...models.http_validation_error import HTTPValidationError
from ...models.metric_history_response import MetricHistoryResponse
from ...models.metric_type import MetricType
from ...models.time_range import TimeRange
from ...types import UNSET, Response, Unset


def _get_kwargs(
    metric_type: MetricType,
    *,
    time_range: TimeRange | Unset = UNSET,
    granularity: str | Unset = "daily",
    force_refresh: bool | Unset = False,
) -> dict[str, Any]:

    params: dict[str, Any] = {}

    json_time_range: str | Unset = UNSET
    if not isinstance(time_range, Unset):
        json_time_range = time_range.value

    params["time_range"] = json_time_range

    params["granularity"] = granularity

    params["force_refresh"] = force_refresh

    params = {k: v for k, v in params.items() if v is not UNSET and v is not None}

    _kwargs: dict[str, Any] = {
        "method": "get",
        "url": "/v1/dashboard/metrics/{metric_type}/history".format(
            metric_type=quote(str(metric_type), safe=""),
        ),
        "params": params,
    }

    return _kwargs


def _parse_response(
    *, client: AuthenticatedClient | Client, response: httpx.Response
) -> HTTPValidationError | MetricHistoryResponse | None:
    if response.status_code == 200:
        response_200 = MetricHistoryResponse.from_dict(response.json())

        return response_200

    if response.status_code == 422:
        response_422 = HTTPValidationError.from_dict(response.json())

        return response_422

    if client.raise_on_unexpected_status:
        raise errors.UnexpectedStatus(response.status_code, response.content)
    else:
        return None


def _build_response(
    *, client: AuthenticatedClient | Client, response: httpx.Response
) -> Response[HTTPValidationError | MetricHistoryResponse]:
    return Response(
        status_code=HTTPStatus(response.status_code),
        content=response.content,
        headers=response.headers,
        parsed=_parse_response(client=client, response=response),
    )


def sync_detailed(
    metric_type: MetricType,
    *,
    client: AuthenticatedClient,
    time_range: TimeRange | Unset = UNSET,
    granularity: str | Unset = "daily",
    force_refresh: bool | Unset = False,
) -> Response[HTTPValidationError | MetricHistoryResponse]:
    """Get Metric History

     Get historical data for a specific metric.

    Useful for detailed analysis of individual metrics over time.
    Data is served from MongoDB cache for faster response times.

    Args:
        metric_type (MetricType):
        time_range (TimeRange | Unset):
        granularity (str | Unset): Data granularity: daily, weekly, monthly Default: 'daily'.
        force_refresh (bool | Unset): Force refresh cache Default: False.

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[HTTPValidationError | MetricHistoryResponse]
    """

    kwargs = _get_kwargs(
        metric_type=metric_type,
        time_range=time_range,
        granularity=granularity,
        force_refresh=force_refresh,
    )

    response = client.get_httpx_client().request(
        **kwargs,
    )

    return _build_response(client=client, response=response)


def sync(
    metric_type: MetricType,
    *,
    client: AuthenticatedClient,
    time_range: TimeRange | Unset = UNSET,
    granularity: str | Unset = "daily",
    force_refresh: bool | Unset = False,
) -> HTTPValidationError | MetricHistoryResponse | None:
    """Get Metric History

     Get historical data for a specific metric.

    Useful for detailed analysis of individual metrics over time.
    Data is served from MongoDB cache for faster response times.

    Args:
        metric_type (MetricType):
        time_range (TimeRange | Unset):
        granularity (str | Unset): Data granularity: daily, weekly, monthly Default: 'daily'.
        force_refresh (bool | Unset): Force refresh cache Default: False.

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        HTTPValidationError | MetricHistoryResponse
    """

    return sync_detailed(
        metric_type=metric_type,
        client=client,
        time_range=time_range,
        granularity=granularity,
        force_refresh=force_refresh,
    ).parsed


async def asyncio_detailed(
    metric_type: MetricType,
    *,
    client: AuthenticatedClient,
    time_range: TimeRange | Unset = UNSET,
    granularity: str | Unset = "daily",
    force_refresh: bool | Unset = False,
) -> Response[HTTPValidationError | MetricHistoryResponse]:
    """Get Metric History

     Get historical data for a specific metric.

    Useful for detailed analysis of individual metrics over time.
    Data is served from MongoDB cache for faster response times.

    Args:
        metric_type (MetricType):
        time_range (TimeRange | Unset):
        granularity (str | Unset): Data granularity: daily, weekly, monthly Default: 'daily'.
        force_refresh (bool | Unset): Force refresh cache Default: False.

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[HTTPValidationError | MetricHistoryResponse]
    """

    kwargs = _get_kwargs(
        metric_type=metric_type,
        time_range=time_range,
        granularity=granularity,
        force_refresh=force_refresh,
    )

    response = await client.get_async_httpx_client().request(**kwargs)

    return _build_response(client=client, response=response)


async def asyncio(
    metric_type: MetricType,
    *,
    client: AuthenticatedClient,
    time_range: TimeRange | Unset = UNSET,
    granularity: str | Unset = "daily",
    force_refresh: bool | Unset = False,
) -> HTTPValidationError | MetricHistoryResponse | None:
    """Get Metric History

     Get historical data for a specific metric.

    Useful for detailed analysis of individual metrics over time.
    Data is served from MongoDB cache for faster response times.

    Args:
        metric_type (MetricType):
        time_range (TimeRange | Unset):
        granularity (str | Unset): Data granularity: daily, weekly, monthly Default: 'daily'.
        force_refresh (bool | Unset): Force refresh cache Default: False.

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        HTTPValidationError | MetricHistoryResponse
    """

    return (
        await asyncio_detailed(
            metric_type=metric_type,
            client=client,
            time_range=time_range,
            granularity=granularity,
            force_refresh=force_refresh,
        )
    ).parsed



from __future__ import annotations

from collections.abc import Mapping
from typing import TYPE_CHECKING, Any, TypeVar, cast

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.action_result_input_type_0 import ActionResultInputType0
    from ..models.action_result_output_type_0 import ActionResultOutputType0


T = TypeVar("T", bound="ActionResult")


@_attrs_define
class ActionResult:
    """
    Attributes:
        input_ (ActionResultInputType0 | None | Unset):
        output (ActionResultOutputType0 | None | Unset):
    """

    input_: ActionResultInputType0 | None | Unset = UNSET
    output: ActionResultOutputType0 | None | Unset = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        from ..models.action_result_input_type_0 import ActionResultInputType0
        from ..models.action_result_output_type_0 import ActionResultOutputType0

        input_: dict[str, Any] | None | Unset
        if isinstance(self.input_, Unset):
            input_ = UNSET
        elif isinstance(self.input_, ActionResultInputType0):
            input_ = self.input_.to_dict()
        else:
            input_ = self.input_

        output: dict[str, Any] | None | Unset
        if isinstance(self.output, Unset):
            output = UNSET
        elif isinstance(self.output, ActionResultOutputType0):
            output = self.output.to_dict()
        else:
            output = self.output

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if input_ is not UNSET:
            field_dict["input"] = input_
        if output is not UNSET:
            field_dict["output"] = output

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        from ..models.action_result_input_type_0 import ActionResultInputType0
        from ..models.action_result_output_type_0 import ActionResultOutputType0

        d = dict(src_dict)

        def _parse_input_(data: object) -> ActionResultInputType0 | None | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                input_type_0 = ActionResultInputType0.from_dict(data)

                return input_type_0
            except (TypeError, ValueError, AttributeError, KeyError):
                pass
            return cast(ActionResultInputType0 | None | Unset, data)

        input_ = _parse_input_(d.pop("input", UNSET))

        def _parse_output(data: object) -> ActionResultOutputType0 | None | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                output_type_0 = ActionResultOutputType0.from_dict(data)

                return output_type_0
            except (TypeError, ValueError, AttributeError, KeyError):
                pass
            return cast(ActionResultOutputType0 | None | Unset, data)

        output = _parse_output(d.pop("output", UNSET))

        action_result = cls(
            input_=input_,
            output=output,
        )

        action_result.additional_properties = d
        return action_result

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from __future__ import annotations

from collections.abc import Mapping
from typing import Any, TypeVar

from attrs import define as _attrs_define
from attrs import field as _attrs_field

T = TypeVar("T", bound="DepartmentMetrics")


@_attrs_define
class DepartmentMetrics:
    """Department-wise metrics

    Attributes:
        department_name (str):
        total_agents (int):
        active_workflows (int):
        time_saved_hours (float):
        dollars_saved (float):
        roi_percentage (float):
    """

    department_name: str
    total_agents: int
    active_workflows: int
    time_saved_hours: float
    dollars_saved: float
    roi_percentage: float
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        department_name = self.department_name

        total_agents = self.total_agents

        active_workflows = self.active_workflows

        time_saved_hours = self.time_saved_hours

        dollars_saved = self.dollars_saved

        roi_percentage = self.roi_percentage

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "department_name": department_name,
                "total_agents": total_agents,
                "active_workflows": active_workflows,
                "time_saved_hours": time_saved_hours,
                "dollars_saved": dollars_saved,
                "roi_percentage": roi_percentage,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        d = dict(src_dict)
        department_name = d.pop("department_name")

        total_agents = d.pop("total_agents")

        active_workflows = d.pop("active_workflows")

        time_saved_hours = d.pop("time_saved_hours")

        dollars_saved = d.pop("dollars_saved")

        roi_percentage = d.pop("roi_percentage")

        department_metrics = cls(
            department_name=department_name,
            total_agents=total_agents,
            active_workflows=active_workflows,
            time_saved_hours=time_saved_hours,
            dollars_saved=dollars_saved,
            roi_percentage=roi_percentage,
        )

        department_metrics.additional_properties = d
        return department_metrics

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from __future__ import annotations

import datetime
from collections.abc import Mapping
from typing import Any, TypeVar, cast

from attrs import define as _attrs_define
from attrs import field as _attrs_field
from dateutil.parser import isoparse

from ..models.knowledge_base_role import KnowledgeBaseRole
from ..types import UNSET, Unset

T = TypeVar("T", bound="KnowledgeBaseMembershipResponse")


@_attrs_define
class KnowledgeBaseMembershipResponse:
    """Membership enriched with user details.

    Attributes:
        field_id (str):
        kb_id (str):
        user_id (str):
        role (KnowledgeBaseRole): Role within a knowledge base / workspace.
        created_at (datetime.datetime | Unset):
        updated_at (datetime.datetime | Unset):
        email (None | str | Unset):
        is_owner (bool | Unset):  Default: False.
    """

    field_id: str
    kb_id: str
    user_id: str
    role: KnowledgeBaseRole
    created_at: datetime.datetime | Unset = UNSET
    updated_at: datetime.datetime | Unset = UNSET
    email: None | str | Unset = UNSET
    is_owner: bool | Unset = False
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        field_id = self.field_id

        kb_id = self.kb_id

        user_id = self.user_id

        role = self.role.value

        created_at: str | Unset = UNSET
        if not isinstance(self.created_at, Unset):
            created_at = self.created_at.isoformat()

        updated_at: str | Unset = UNSET
        if not isinstance(self.updated_at, Unset):
            updated_at = self.updated_at.isoformat()

        email: None | str | Unset
        if isinstance(self.email, Unset):
            email = UNSET
        else:
            email = self.email

        is_owner = self.is_owner

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "_id": field_id,
                "kb_id": kb_id,
                "user_id": user_id,
                "role": role,
            }
        )
        if created_at is not UNSET:
            field_dict["created_at"] = created_at
        if updated_at is not UNSET:
            field_dict["updated_at"] = updated_at
        if email is not UNSET:
            field_dict["email"] = email
        if is_owner is not UNSET:
            field_dict["is_owner"] = is_owner

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        d = dict(src_dict)
        field_id = d.pop("_id")

        kb_id = d.pop("kb_id")

        user_id = d.pop("user_id")

        role = KnowledgeBaseRole(d.pop("role"))

        _created_at = d.pop("created_at", UNSET)
        created_at: datetime.datetime | Unset
        if isinstance(_created_at, Unset):
            created_at = UNSET
        else:
            created_at = isoparse(_created_at)

        _updated_at = d.pop("updated_at", UNSET)
        updated_at: datetime.datetime | Unset
        if isinstance(_updated_at, Unset):
            updated_at = UNSET
        else:
            updated_at = isoparse(_updated_at)

        def _parse_email(data: object) -> None | str | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(None | str | Unset, data)

        email = _parse_email(d.pop("email", UNSET))

        is_owner = d.pop("is_owner", UNSET)

        knowledge_base_membership_response = cls(
            field_id=field_id,
            kb_id=kb_id,
            user_id=user_id,
            role=role,
            created_at=created_at,
            updated_at=updated_at,
            email=email,
            is_owner=is_owner,
        )

        knowledge_base_membership_response.additional_properties = d
        return knowledge_base_membership_response

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from __future__ import annotations

from collections.abc import Mapping
from typing import Any, TypeVar, cast

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

T = TypeVar("T", bound="DatabaseConnection")


@_attrs_define
class DatabaseConnection:
    """Response model for database connection info (without sensitive data)

    Attributes:
        id (str):
        name (None | str):
        database_type (str):
        host (None | str | Unset):
        port (int | None | Unset):
        database (None | str | Unset):
        username (None | str | Unset):
        project_id (None | str | Unset):
        dataset (None | str | Unset):
    """

    id: str
    name: None | str
    database_type: str
    host: None | str | Unset = UNSET
    port: int | None | Unset = UNSET
    database: None | str | Unset = UNSET
    username: None | str | Unset = UNSET
    project_id: None | str | Unset = UNSET
    dataset: None | str | Unset = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        id = self.id

        name: None | str
        name = self.name

        database_type = self.database_type

        host: None | str | Unset
        if isinstance(self.host, Unset):
            host = UNSET
        else:
            host = self.host

        port: int | None | Unset
        if isinstance(self.port, Unset):
            port = UNSET
        else:
            port = self.port

        database: None | str | Unset
        if isinstance(self.database, Unset):
            database = UNSET
        else:
            database = self.database

        username: None | str | Unset
        if isinstance(self.username, Unset):
            username = UNSET
        else:
            username = self.username

        project_id: None | str | Unset
        if isinstance(self.project_id, Unset):
            project_id = UNSET
        else:
            project_id = self.project_id

        dataset: None | str | Unset
        if isinstance(self.dataset, Unset):
            dataset = UNSET
        else:
            dataset = self.dataset

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "id": id,
                "name": name,
                "database_type": database_type,
            }
        )
        if host is not UNSET:
            field_dict["host"] = host
        if port is not UNSET:
            field_dict["port"] = port
        if database is not UNSET:
            field_dict["database"] = database
        if username is not UNSET:
            field_dict["username"] = username
        if project_id is not UNSET:
            field_dict["project_id"] = project_id
        if dataset is not UNSET:
            field_dict["dataset"] = dataset

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        d = dict(src_dict)
        id = d.pop("id")

        def _parse_name(data: object) -> None | str:
            if data is None:
                return data
            return cast(None | str, data)

        name = _parse_name(d.pop("name"))

        database_type = d.pop("database_type")

        def _parse_host(data: object) -> None | str | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(None | str | Unset, data)

        host = _parse_host(d.pop("host", UNSET))

        def _parse_port(data: object) -> int | None | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(int | None | Unset, data)

        port = _parse_port(d.pop("port", UNSET))

        def _parse_database(data: object) -> None | str | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(None | str | Unset, data)

        database = _parse_database(d.pop("database", UNSET))

        def _parse_username(data: object) -> None | str | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(None | str | Unset, data)

        username = _parse_username(d.pop("username", UNSET))

        def _parse_project_id(data: object) -> None | str | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(None | str | Unset, data)

        project_id = _parse_project_id(d.pop("project_id", UNSET))

        def _parse_dataset(data: object) -> None | str | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(None | str | Unset, data)

        dataset = _parse_dataset(d.pop("dataset", UNSET))

        database_connection = cls(
            id=id,
            name=name,
            database_type=database_type,
            host=host,
            port=port,
            database=database,
            username=username,
            project_id=project_id,
            dataset=dataset,
        )

        database_connection.additional_properties = d
        return database_connection

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



"""Contains all the data models used in inputs/outputs"""

from .action_create import ActionCreate
from .action_create_audio_2_text_transcription_openai import ActionCreateAudio2TextTranscriptionOpenai
from .action_create_audio_2_text_translation_openai import ActionCreateAudio2TextTranslationOpenai
from .action_create_config import ActionCreateConfig
from .action_create_img_2_img_edit_openai import ActionCreateImg2ImgEditOpenai
from .action_create_img_2_img_variation_openai import ActionCreateImg2ImgVariationOpenai
from .action_create_input_type_0 import ActionCreateInputType0
from .action_create_output_type_0 import ActionCreateOutputType0
from .action_create_text_2_audio_speech_openai import ActionCreateText2AudioSpeechOpenai
from .action_create_text_2_text_llm_chat_openai import ActionCreateText2TextLlmChatOpenai
from .action_create_text_2_text_read_url import ActionCreateText2TextReadUrl
from .action_create_text_2_text_read_url_input_type_0 import ActionCreateText2TextReadUrlInputType0
from .action_create_text_2_text_read_url_output_type_0 import ActionCreateText2TextReadUrlOutputType0
from .action_create_text_2_text_search_web import ActionCreateText2TextSearchWeb
from .action_create_text_2_text_search_web_input_type_0 import ActionCreateText2TextSearchWebInputType0
from .action_create_text_2_text_search_web_output_type_0 import ActionCreateText2TextSearchWebOutputType0
from .action_create_text_2_text_user_input_input_type_0 import ActionCreateText2TextUserInputInputType0
from .action_create_text_2_text_user_input_output_type_0 import ActionCreateText2TextUserInputOutputType0
from .action_doc import ActionDoc
from .action_doc_config import ActionDocConfig
from .action_doc_input_type_0 import ActionDocInputType0
from .action_doc_output_type_0 import ActionDocOutputType0
from .action_docs_found import ActionDocsFound
from .action_graph_create import ActionGraphCreate
from .action_graph_create_graph_type_0 import ActionGraphCreateGraphType0
from .action_graph_create_input_type_0 import ActionGraphCreateInputType0
from .action_graph_create_node_details import ActionGraphCreateNodeDetails
from .action_graph_create_nodes_type_0 import ActionGraphCreateNodesType0
from .action_graph_create_output_type_0 import ActionGraphCreateOutputType0
from .action_graph_doc import ActionGraphDoc
from .action_graph_doc_graph_type_0 import ActionGraphDocGraphType0
from .action_graph_doc_input_type_0 import ActionGraphDocInputType0
from .action_graph_doc_node_details import ActionGraphDocNodeDetails
from .action_graph_doc_nodes_type_0 import ActionGraphDocNodesType0
from .action_graph_doc_output_type_0 import ActionGraphDocOutputType0
from .action_graph_docs_found import ActionGraphDocsFound
from .action_graph_lite_doc import ActionGraphLiteDoc
from .action_graph_result_docs_found import ActionGraphResultDocsFound
from .action_graph_result_lite_doc import ActionGraphResultLiteDoc
from .action_graph_result_update import ActionGraphResultUpdate
from .action_graph_update import ActionGraphUpdate
from .action_graph_update_graph_type_0 import ActionGraphUpdateGraphType0
from .action_graph_update_node_details_type_0 import ActionGraphUpdateNodeDetailsType0
from .action_graph_update_nodes_type_0 import ActionGraphUpdateNodesType0
from .action_graph_update_output_type_0 import ActionGraphUpdateOutputType0
from .action_lite_doc import ActionLiteDoc
from .action_result import ActionResult
from .action_result_doc import ActionResultDoc
from .action_result_docs_found import ActionResultDocsFound
from .action_result_input_type_0 import ActionResultInputType0
from .action_result_lite_doc import ActionResultLiteDoc
from .action_result_output_type_0 import ActionResultOutputType0
from .action_result_update import ActionResultUpdate
from .action_type import ActionType
from .action_update import ActionUpdate
from .action_update_config_type_0 import ActionUpdateConfigType0
from .adp_client_credentials_request import ADPClientCredentialsRequest
from .agent_data import AgentData
from .agent_data_openai_provider import AgentDataOpenaiProvider
from .agent_performance import AgentPerformance
from .app_auth_lite_doc import AppAuthLiteDoc
from .app_auth_lite_doc_page import AppAuthLiteDocPage
from .app_auth_read import AppAuthRead
from .app_creds_create import AppCredsCreate
from .app_creds_create_creds import AppCredsCreateCreds
from .app_creds_doc import AppCredsDoc
from .app_creds_doc_creds import AppCredsDocCreds
from .app_creds_lite_doc import AppCredsLiteDoc
from .app_creds_lite_doc_page import AppCredsLiteDocPage
from .app_creds_read import AppCredsRead
from .app_creds_update import AppCredsUpdate
from .app_creds_update_creds import AppCredsUpdateCreds
from .app_types import AppTypes
from .audio import Audio
from .audio_res import AudioRes
from .auth_response import AuthResponse
from .auto_file_chunking_strategy_param import AutoFileChunkingStrategyParam
from .base_model import BaseModel
from .bearer_access_token import BearerAccessToken
from .body_async_run_action_loop_v1_actions_id_async_run_loop_post import (
    BodyAsyncRunActionLoopV1ActionsIdAsyncRunLoopPost,
)
from .body_async_run_action_loop_v1_actions_id_async_run_loop_post_input_item import (
    BodyAsyncRunActionLoopV1ActionsIdAsyncRunLoopPostInputItem,
)
from .body_async_run_action_v1_actions_id_async_run_post import BodyAsyncRunActionV1ActionsIdAsyncRunPost
from .body_async_run_action_v1_actions_id_async_run_post_input import BodyAsyncRunActionV1ActionsIdAsyncRunPostInput
from .body_cookie_cookie_post import BodyCookieCookiePost
from .body_ingest_file_async_v1_ingestion_file_async_post import BodyIngestFileAsyncV1IngestionFileAsyncPost
from .body_process_password_reset_password_reset_post import BodyProcessPasswordResetPasswordResetPost
from .body_return_token_v1_auth_token_post import BodyReturnTokenV1AuthTokenPost
from .body_return_user_and_session_v1_auth_post import BodyReturnUserAndSessionV1AuthPost
from .body_run_slack_cmd_v1_webhooks_slack_command_post import BodyRunSlackCmdV1WebhooksSlackCommandPost
from .body_signup_signup_post import BodySignupSignupPost
from .body_upload_file_on_azure_v1_openai_storage_files_azure_post import (
    BodyUploadFileOnAzureV1OpenaiStorageFilesAzurePost,
)
from .body_upload_file_on_openai_v1_openai_storage_files_openai_post import (
    BodyUploadFileOnOpenaiV1OpenaiStorageFilesOpenaiPost,
)
from .body_upload_file_v1_openai_storage_files_post import BodyUploadFileV1OpenaiStorageFilesPost
from .body_upload_files_v1_files_post import BodyUploadFilesV1FilesPost
from .body_upload_public_files_v1_files_public_post import BodyUploadPublicFilesV1FilesPublicPost
from .body_v2_ingest_file_into_collection_v1_vectorstore_v2_collections_collection_name_files_async_post import (
    BodyV2IngestFileIntoCollectionV1VectorstoreV2CollectionsCollectionNameFilesAsyncPost,
)
from .bulk_member_response import BulkMemberResponse
from .character_create import CharacterCreate
from .character_doc import CharacterDoc
from .character_lite_doc import CharacterLiteDoc
from .character_lite_doc_page import CharacterLiteDocPage
from .character_update import CharacterUpdate
from .chart_data import ChartData
from .chat_completion_allowed_tool_choice_param import ChatCompletionAllowedToolChoiceParam
from .chat_completion_allowed_tools_param import ChatCompletionAllowedToolsParam
from .chat_completion_allowed_tools_param_mode import ChatCompletionAllowedToolsParamMode
from .chat_completion_allowed_tools_param_tools_item import ChatCompletionAllowedToolsParamToolsItem
from .chat_completion_assistant_message_param import ChatCompletionAssistantMessageParam
from .chat_completion_audio_param import ChatCompletionAudioParam
from .chat_completion_audio_param_format import ChatCompletionAudioParamFormat
from .chat_completion_audio_param_voice_type_1 import ChatCompletionAudioParamVoiceType1
from .chat_completion_content_part_image_param import ChatCompletionContentPartImageParam
from .chat_completion_content_part_input_audio_param import ChatCompletionContentPartInputAudioParam
from .chat_completion_content_part_refusal_param import ChatCompletionContentPartRefusalParam
from .chat_completion_content_part_text_param import ChatCompletionContentPartTextParam
from .chat_completion_developer_message_param import ChatCompletionDeveloperMessageParam
from .chat_completion_function_message_param import ChatCompletionFunctionMessageParam
from .chat_completion_prediction_content_param import ChatCompletionPredictionContentParam
from .chat_completion_stream_options_param import ChatCompletionStreamOptionsParam
from .chat_completion_system_message_param import ChatCompletionSystemMessageParam
from .chat_completion_tool_message_param import ChatCompletionToolMessageParam
from .chat_completion_user_message_param import ChatCompletionUserMessageParam
from .chat_doc import ChatDoc
from .chat_doc_openai_provider import ChatDocOpenaiProvider
from .chat_update import ChatUpdate
from .clone_action_request import CloneActionRequest
from .clone_action_result import CloneActionResult
from .clone_action_v1_actions_clone_post_response_clone_action_v1_actions_clone_post import (
    CloneActionV1ActionsClonePostResponseCloneActionV1ActionsClonePost,
)
from .code_interpreter_tool import CodeInterpreterTool
from .code_interpreter_tool_param import CodeInterpreterToolParam
from .cost_config_create import CostConfigCreate
from .cost_config_list_response import CostConfigListResponse
from .cost_config_response import CostConfigResponse
from .cost_config_type import CostConfigType
from .cost_config_update import CostConfigUpdate
from .create_chat_v1_action_chats_post_openai_provider import CreateChatV1ActionChatsPostOpenaiProvider
from .create_vector_store import CreateVectorStore
from .create_vector_store_file import CreateVectorStoreFile
from .create_vector_store_file_batch import CreateVectorStoreFileBatch
from .create_vector_store_files_v1_openai_storage_vector_store_file_batches_post_openai_provider import (
    CreateVectorStoreFilesV1OpenaiStorageVectorStoreFileBatchesPostOpenaiProvider,
)
from .create_vector_store_files_v1_openai_storage_vector_store_files_post_openai_provider import (
    CreateVectorStoreFilesV1OpenaiStorageVectorStoreFilesPostOpenaiProvider,
)
from .create_vector_store_metadata_type_0 import CreateVectorStoreMetadataType0
from .create_vector_store_v1_openai_storage_vector_stores_post_openai_provider import (
    CreateVectorStoreV1OpenaiStorageVectorStoresPostOpenaiProvider,
)
from .cron_schedule import CronSchedule
from .cron_schedule_day_mode import CronScheduleDayMode
from .cron_schedule_day_of_week_type_0_item import CronScheduleDayOfWeekType0Item
from .cron_schedule_day_of_week_type_1 import CronScheduleDayOfWeekType1
from .cron_schedule_hour_mode import CronScheduleHourMode
from .cron_schedule_minute_mode import CronScheduleMinuteMode
from .cron_schedule_month_mode import CronScheduleMonthMode
from .cron_schedule_week_mode import CronScheduleWeekMode
from .custom import Custom
from .dalle_openai_config import DalleOpenaiConfig
from .dalle_openai_config_provider import DalleOpenaiConfigProvider
from .dalle_openai_config_quality import DalleOpenaiConfigQuality
from .dalle_openai_config_response_format_type_0 import DalleOpenaiConfigResponseFormatType0
from .dalle_openai_config_size_type_0 import DalleOpenaiConfigSizeType0
from .dalle_openai_config_style_type_0 import DalleOpenaiConfigStyleType0
from .dashboard_metric import DashboardMetric
from .dashboard_request import DashboardRequest
from .dashboard_response import DashboardResponse
from .dashboard_summary import DashboardSummary
from .data_flow_type import DataFlowType
from .database_connection import DatabaseConnection
from .database_creds_model import DatabaseCredsModel
from .database_creds_model_credentials_info_type_0 import DatabaseCredsModelCredentialsInfoType0
from .database_enum import DatabaseEnum
from .datastore_meta_doc import DatastoreMetaDoc
from .db_auth_doc import DbAuthDoc
from .db_auth_doc_auth import DbAuthDocAuth
from .db_auth_lite_doc import DbAuthLiteDoc
from .db_auth_lite_doc_page import DbAuthLiteDocPage
from .delete_cost_config_v1_dashboard_cost_config_config_type_delete_response_delete_cost_config_v1_dashboard_cost_config_config_type_delete import (
    DeleteCostConfigV1DashboardCostConfigConfigTypeDeleteResponseDeleteCostConfigV1DashboardCostConfigConfigTypeDelete,
)
from .delete_file_v1_openai_storage_files_delete_openai_provider import (
    DeleteFileV1OpenaiStorageFilesDeleteOpenaiProvider,
)
from .delete_vector_store import DeleteVectorStore
from .delete_vector_store_file import DeleteVectorStoreFile
from .delete_vector_store_files_v1_openai_storage_vector_store_file_batches_delete_openai_provider import (
    DeleteVectorStoreFilesV1OpenaiStorageVectorStoreFileBatchesDeleteOpenaiProvider,
)
from .delete_vector_store_v1_openai_storage_vector_stores_delete_openai_provider import (
    DeleteVectorStoreV1OpenaiStorageVectorStoresDeleteOpenaiProvider,
)
from .deleted import Deleted
from .department_metrics import DepartmentMetrics
from .doc_find_page import DocFindPage
from .event_result_status import EventResultStatus
from .event_type import EventType
from .expires_after import ExpiresAfter
from .factor import Factor
from .factor_factor_type_type_0 import FactorFactorTypeType0
from .factor_status import FactorStatus
from .file import File
from .file_and_processing_params import FileAndProcessingParams
from .file_counts import FileCounts
from .file_deleted import FileDeleted
from .file_file import FileFile
from .file_list_response_schema import FileListResponseSchema
from .file_meta import FileMeta
from .file_object import FileObject
from .file_object_purpose import FileObjectPurpose
from .file_object_status import FileObjectStatus
from .file_params import FileParams
from .file_search import FileSearch
from .file_search_ranking_options import FileSearchRankingOptions
from .file_search_ranking_options_ranker_type_0 import FileSearchRankingOptionsRankerType0
from .files_and_processing_params import FilesAndProcessingParams
from .filter_by import FilterBy
from .function import Function
from .function_call import FunctionCall
from .function_definition import FunctionDefinition
from .function_definition_parameters_type_0 import FunctionDefinitionParametersType0
from .get_creds_types_v1_app_creds_types_app_get_response_get_creds_types_v1_app_creds_types_app_get import (
    GetCredsTypesV1AppCredsTypesAppGetResponseGetCredsTypesV1AppCredsTypesAppGet,
)
from .get_file_info_v1_openai_storage_files_get_openai_provider import GetFileInfoV1OpenaiStorageFilesGetOpenaiProvider
from .get_file_v1_openai_storage_files_content_get_openai_provider import (
    GetFileV1OpenaiStorageFilesContentGetOpenaiProvider,
)
from .get_vector_store_files_v1_openai_storage_vector_store_file_batches_get_openai_provider import (
    GetVectorStoreFilesV1OpenaiStorageVectorStoreFileBatchesGetOpenaiProvider,
)
from .get_vector_store_v1_openai_storage_vector_stores_get_openai_provider import (
    GetVectorStoreV1OpenaiStorageVectorStoresGetOpenaiProvider,
)
from .graph_rag_search_data import GraphRAGSearchData
from .graph_rag_search_data_graph_visualization_type_0 import GraphRAGSearchDataGraphVisualizationType0
from .http_validation_error import HTTPValidationError
from .image import Image
from .image_create_variation import ImageCreateVariation
from .image_create_variation_extra_headers_type_0 import ImageCreateVariationExtraHeadersType0
from .image_create_variation_extra_query_type_0 import ImageCreateVariationExtraQueryType0
from .image_create_variation_response_format_type_0 import ImageCreateVariationResponseFormatType0
from .image_create_variation_size_type_0 import ImageCreateVariationSizeType0
from .image_edit import ImageEdit
from .image_edit_extra_headers_type_0 import ImageEditExtraHeadersType0
from .image_edit_extra_query_type_0 import ImageEditExtraQueryType0
from .image_edit_input import ImageEditInput
from .image_edit_input_size_type_0 import ImageEditInputSizeType0
from .image_edit_response_format_type_0 import ImageEditResponseFormatType0
from .image_edit_size_type_0 import ImageEditSizeType0
from .image_url import ImageURL
from .image_url_detail import ImageURLDetail
from .image_variation_input import ImageVariationInput
from .image_variation_input_size_type_0 import ImageVariationInputSizeType0
from .images_response import ImagesResponse
from .images_response_background_type_0 import ImagesResponseBackgroundType0
from .images_response_output_format_type_0 import ImagesResponseOutputFormatType0
from .images_response_quality_type_0 import ImagesResponseQualityType0
from .images_response_size_type_0 import ImagesResponseSizeType0
from .ingest_async_response import IngestAsyncResponse
from .initialize_default_configs_v1_dashboard_cost_config_initialize_post_response_initialize_default_configs_v1_dashboard_cost_config_initialize_post import (
    InitializeDefaultConfigsV1DashboardCostConfigInitializePostResponseInitializeDefaultConfigsV1DashboardCostConfigInitializePost,
)
from .input_audio import InputAudio
from .input_audio_format import InputAudioFormat
from .interval_trigger_model import IntervalTriggerModel
from .job_status_schema import JobStatusSchema
from .json_schema import JSONSchema
from .json_schema_schema import JSONSchemaSchema
from .knowledge_base_create_request import KnowledgeBaseCreateRequest
from .knowledge_base_member_by_email_request import KnowledgeBaseMemberByEmailRequest
from .knowledge_base_member_create_request import KnowledgeBaseMemberCreateRequest
from .knowledge_base_member_update_request import KnowledgeBaseMemberUpdateRequest
from .knowledge_base_members_bulk_request import KnowledgeBaseMembersBulkRequest
from .knowledge_base_membership_response import KnowledgeBaseMembershipResponse
from .knowledge_base_response import KnowledgeBaseResponse
from .knowledge_base_role import KnowledgeBaseRole
from .knowledge_base_update_request import KnowledgeBaseUpdateRequest
from .last_error import LastError
from .last_error_code import LastErrorCode
from .list_action_tools_v1_actions_tools_get_llm_type import ListActionToolsV1ActionsToolsGetLlmType
from .list_files_info_v1_openai_storage_files_list_get_openai_provider import (
    ListFilesInfoV1OpenaiStorageFilesListGetOpenaiProvider,
)
from .list_ingested_files_proxy_v1_ingestion_files_get_scope_type_0 import (
    ListIngestedFilesProxyV1IngestionFilesGetScopeType0,
)
from .list_ingested_files_proxy_v1_ingestion_files_get_sort_by import ListIngestedFilesProxyV1IngestionFilesGetSortBy
from .list_ingested_files_proxy_v1_ingestion_files_get_sort_order import (
    ListIngestedFilesProxyV1IngestionFilesGetSortOrder,
)
from .list_mcps_v1_mcps_get_request_headers import ListMcpsV1McpsGetRequestHeaders
from .list_mcps_v1_mcps_get_server_type import ListMcpsV1McpsGetServerType
from .list_vector_store_files_v1_openai_storage_vector_store_file_batches_list_get_filter_type_0 import (
    ListVectorStoreFilesV1OpenaiStorageVectorStoreFileBatchesListGetFilterType0,
)
from .list_vector_store_files_v1_openai_storage_vector_store_file_batches_list_get_openai_provider import (
    ListVectorStoreFilesV1OpenaiStorageVectorStoreFileBatchesListGetOpenaiProvider,
)
from .list_vector_store_files_v1_openai_storage_vector_store_file_batches_list_get_order_type_0 import (
    ListVectorStoreFilesV1OpenaiStorageVectorStoreFileBatchesListGetOrderType0,
)
from .list_vector_store_files_v1_openai_storage_vector_store_files_list_get_filter_type_0 import (
    ListVectorStoreFilesV1OpenaiStorageVectorStoreFilesListGetFilterType0,
)
from .list_vector_store_files_v1_openai_storage_vector_store_files_list_get_openai_provider import (
    ListVectorStoreFilesV1OpenaiStorageVectorStoreFilesListGetOpenaiProvider,
)
from .list_vector_store_files_v1_openai_storage_vector_store_files_list_get_order_type_0 import (
    ListVectorStoreFilesV1OpenaiStorageVectorStoreFilesListGetOrderType0,
)
from .list_vector_stores_v1_openai_storage_vector_stores_list_get_openai_provider import (
    ListVectorStoresV1OpenaiStorageVectorStoresListGetOpenaiProvider,
)
from .list_vector_stores_v1_openai_storage_vector_stores_list_get_order import (
    ListVectorStoresV1OpenaiStorageVectorStoresListGetOrder,
)
from .llm_chat_openai_config import LlmChatOpenaiConfig
from .llm_chat_openai_config_extra_headers_type_0 import LlmChatOpenaiConfigExtraHeadersType0
from .llm_chat_openai_config_extra_query_type_0 import LlmChatOpenaiConfigExtraQueryType0
from .llm_chat_openai_config_json_schema_type_0 import LlmChatOpenaiConfigJsonSchemaType0
from .llm_chat_openai_config_logit_bias_type_0 import LlmChatOpenaiConfigLogitBiasType0
from .llm_chat_openai_config_metadata_type_0 import LlmChatOpenaiConfigMetadataType0
from .llm_chat_openai_config_modalities_type_0_item import LlmChatOpenaiConfigModalitiesType0Item
from .llm_chat_openai_config_model_type_0 import LlmChatOpenaiConfigModelType0
from .llm_chat_openai_config_model_type_1 import LlmChatOpenaiConfigModelType1
from .llm_chat_openai_config_prompt_cache_retention_type_0 import LlmChatOpenaiConfigPromptCacheRetentionType0
from .llm_chat_openai_config_provider import LlmChatOpenaiConfigProvider
from .llm_chat_openai_config_reasoning_effort_type_0 import LlmChatOpenaiConfigReasoningEffortType0
from .llm_chat_openai_config_response_schema_type_1 import LlmChatOpenaiConfigResponseSchemaType1
from .llm_chat_openai_config_service_tier_type_0 import LlmChatOpenaiConfigServiceTierType0
from .llm_chat_openai_config_tool_choice_type_0 import LlmChatOpenaiConfigToolChoiceType0
from .llm_chat_openai_config_verbosity_type_0 import LlmChatOpenaiConfigVerbosityType0
from .looker_client_credentials_request import LookerClientCredentialsRequest
from .mcp_create import MCPCreate
from .mcp_create_request_headers_type_0 import MCPCreateRequestHeadersType0
from .mcp_create_server_type import MCPCreateServerType
from .mcp_doc import MCPDoc
from .mcp_doc_request_headers_type_0 import MCPDocRequestHeadersType0
from .mcp_doc_server_type import MCPDocServerType
from .mcp_docs_found import MCPDocsFound
from .mcp_update import MCPUpdate
from .mcp_update_request_headers_type_0 import MCPUpdateRequestHeadersType0
from .mcp_update_server_type_type_0 import MCPUpdateServerTypeType0
from .message import Message
from .message_content_type_1_item import MessageContentType1Item
from .metadata import Metadata
from .metric_history_response import MetricHistoryResponse
from .metric_history_response_summary_stats import MetricHistoryResponseSummaryStats
from .metric_type import MetricType
from .net_suite_tba_request import NetSuiteTBARequest
from .node import Node
from .node_data import NodeData
from .other_file_chunking_strategy_object import OtherFileChunkingStrategyObject
from .output_type import OutputType
from .partition_parameters_params import PartitionParametersParams
from .partition_parameters_params_hi_res_model_name_type_0 import PartitionParametersParamsHiResModelNameType0
from .partition_parameters_params_strategy_type_0 import PartitionParametersParamsStrategyType0
from .position import Position
from .prompt_update import PromptUpdate
from .read_url_config import ReadUrlConfig
from .region import Region
from .response_format_json_object import ResponseFormatJSONObject
from .response_format_json_schema import ResponseFormatJSONSchema
from .response_format_text import ResponseFormatText
from .retrieve_vector_store import RetrieveVectorStore
from .retrieve_vector_store_file_batch import RetrieveVectorStoreFileBatch
from .role import Role
from .run_action_stream_v1_actions_id_run_stream_post_input import RunActionStreamV1ActionsIdRunStreamPostInput
from .run_action_v1_actions_id_run_post_input import RunActionV1ActionsIdRunPostInput
from .run_react_agent_v1_agents_react_post_openai_provider import RunReactAgentV1AgentsReactPostOpenaiProvider
from .run_webhook_event_v1_webhooks_event_id_post_event import RunWebhookEventV1WebhooksEventIdPostEvent
from .runnable_type import RunnableType
from .schedule_update import ScheduleUpdate
from .search_result import SearchResult
from .search_result_metadata_type_0 import SearchResultMetadataType0
from .search_text_params import SearchTextParams
from .search_text_params_safesearch import SearchTextParamsSafesearch
from .search_web_config import SearchWebConfig
from .search_web_config_search_type import SearchWebConfigSearchType
from .session import Session
from .share_data_create import ShareDataCreate
from .share_data_doc import ShareDataDoc
from .share_data_docs_page import ShareDataDocsPage
from .share_data_update import ShareDataUpdate
from .siri_automation_payload import SiriAutomationPayload
from .source_chunk import SourceChunk
from .source_chunk_metadata import SourceChunkMetadata
from .speech_req import SpeechReq
from .speech_req_model import SpeechReqModel
from .speech_req_response_format import SpeechReqResponseFormat
from .speech_req_voice import SpeechReqVoice
from .spotify_client_credentials_request import SpotifyClientCredentialsRequest
from .static_file_chunking_strategy import StaticFileChunkingStrategy
from .static_file_chunking_strategy_object import StaticFileChunkingStrategyObject
from .static_file_chunking_strategy_object_param import StaticFileChunkingStrategyObjectParam
from .static_file_chunking_strategy_param import StaticFileChunkingStrategyParam
from .structured_output_schema import StructuredOutputSchema
from .structured_output_schema_properties import StructuredOutputSchemaProperties
from .sync_app_auth_v1_app_auth_sync_get_response_sync_app_auth_v1_app_auth_sync_get import (
    SyncAppAuthV1AppAuthSyncGetResponseSyncAppAuthV1AppAuthSyncGet,
)
from .team_create import TeamCreate
from .team_doc import TeamDoc
from .team_lite_doc import TeamLiteDoc
from .team_lite_doc_page import TeamLiteDocPage
from .team_update import TeamUpdate
from .test_db_connection_v1_db_auth_test_connection_db_auth_id_post_response_test_db_connection_v1_db_auth_test_connection_db_auth_id_post import (
    TestDbConnectionV1DbAuthTestConnectionDbAuthIdPostResponseTestDbConnectionV1DbAuthTestConnectionDbAuthIdPost,
)
from .text_obj import TextObj
from .text_objs import TextObjs
from .time_range import TimeRange
from .time_range_config import TimeRangeConfig
from .time_series_data_point import TimeSeriesDataPoint
from .timelimit import Timelimit
from .timezone_enum import TimezoneEnum
from .tool_resources import ToolResources
from .tool_resources_code_interpreter import ToolResourcesCodeInterpreter
from .tool_resources_file_search import ToolResourcesFileSearch
from .tool_resources_file_search_vector_store import ToolResourcesFileSearchVectorStore
from .tool_resources_file_search_vector_store_chunking_strategy_auto import (
    ToolResourcesFileSearchVectorStoreChunkingStrategyAuto,
)
from .tool_resources_file_search_vector_store_chunking_strategy_static import (
    ToolResourcesFileSearchVectorStoreChunkingStrategyStatic,
)
from .tool_resources_file_search_vector_store_chunking_strategy_static_static import (
    ToolResourcesFileSearchVectorStoreChunkingStrategyStaticStatic,
)
from .tool_resources_file_search_vector_store_metadata_type_0 import ToolResourcesFileSearchVectorStoreMetadataType0
from .transcription_req import TranscriptionReq
from .transcription_req_response_format import TranscriptionReqResponseFormat
from .translation_req import TranslationReq
from .translation_req_response_format import TranslationReqResponseFormat
from .update_vector_store import UpdateVectorStore
from .update_vector_store_v1_openai_storage_vector_stores_put_openai_provider import (
    UpdateVectorStoreV1OpenaiStorageVectorStoresPutOpenaiProvider,
)
from .url_obj import UrlObj
from .url_response import UrlResponse
from .usage import Usage
from .usage_input_tokens_details import UsageInputTokensDetails
from .usage_output_tokens_details import UsageOutputTokensDetails
from .user import User
from .user_app_metadata import UserAppMetadata
from .user_attributes import UserAttributes
from .user_identity import UserIdentity
from .user_identity_identity_data import UserIdentityIdentityData
from .user_user_metadata import UserUserMetadata
from .v2_delete_collection_v1_vectorstore_v2_collections_collection_name_delete_response_v2_delete_collection_v1_vectorstore_v2_collections_collection_name_delete import (
    V2DeleteCollectionV1VectorstoreV2CollectionsCollectionNameDeleteResponseV2DeleteCollectionV1VectorstoreV2CollectionsCollectionNameDelete,
)
from .validation_error import ValidationError
from .validation_error_context import ValidationErrorContext
from .vector_store_deleted import VectorStoreDeleted
from .vector_store_file import VectorStoreFile
from .vector_store_file_attributes_type_0 import VectorStoreFileAttributesType0
from .vector_store_file_batch import VectorStoreFileBatch
from .vector_store_file_batch_status import VectorStoreFileBatchStatus
from .vector_store_file_deleted import VectorStoreFileDeleted
from .vector_store_file_status import VectorStoreFileStatus
from .web_search_options import WebSearchOptions
from .web_search_options_search_context_size import WebSearchOptionsSearchContextSize
from .web_search_options_user_location import WebSearchOptionsUserLocation
from .web_search_options_user_location_approximate import WebSearchOptionsUserLocationApproximate
from .webhook import Webhook
from .webhook_create import WebhookCreate
from .webhook_doc import WebhookDoc
from .webhook_update import WebhookUpdate
from .workflow_performance import WorkflowPerformance
from .zendesk_webhook_v1_webhooks_zendesk_post_event import ZendeskWebhookV1WebhooksZendeskPostEvent

__all__ = (
    "ActionCreate",
    "ActionCreateAudio2TextTranscriptionOpenai",
    "ActionCreateAudio2TextTranslationOpenai",
    "ActionCreateConfig",
    "ActionCreateImg2ImgEditOpenai",
    "ActionCreateImg2ImgVariationOpenai",
    "ActionCreateInputType0",
    "ActionCreateOutputType0",
    "ActionCreateText2AudioSpeechOpenai",
    "ActionCreateText2TextLlmChatOpenai",
    "ActionCreateText2TextReadUrl",
    "ActionCreateText2TextReadUrlInputType0",
    "ActionCreateText2TextReadUrlOutputType0",
    "ActionCreateText2TextSearchWeb",
    "ActionCreateText2TextSearchWebInputType0",
    "ActionCreateText2TextSearchWebOutputType0",
    "ActionCreateText2TextUserInputInputType0",
    "ActionCreateText2TextUserInputOutputType0",
    "ActionDoc",
    "ActionDocConfig",
    "ActionDocInputType0",
    "ActionDocOutputType0",
    "ActionDocsFound",
    "ActionGraphCreate",
    "ActionGraphCreateGraphType0",
    "ActionGraphCreateInputType0",
    "ActionGraphCreateNodeDetails",
    "ActionGraphCreateNodesType0",
    "ActionGraphCreateOutputType0",
    "ActionGraphDoc",
    "ActionGraphDocGraphType0",
    "ActionGraphDocInputType0",
    "ActionGraphDocNodeDetails",
    "ActionGraphDocNodesType0",
    "ActionGraphDocOutputType0",
    "ActionGraphDocsFound",
    "ActionGraphLiteDoc",
    "ActionGraphResultDocsFound",
    "ActionGraphResultLiteDoc",
    "ActionGraphResultUpdate",
    "ActionGraphUpdate",
    "ActionGraphUpdateGraphType0",
    "ActionGraphUpdateNodeDetailsType0",
    "ActionGraphUpdateNodesType0",
    "ActionGraphUpdateOutputType0",
    "ActionLiteDoc",
    "ActionResult",
    "ActionResultDoc",
    "ActionResultDocsFound",
    "ActionResultInputType0",
    "ActionResultLiteDoc",
    "ActionResultOutputType0",
    "ActionResultUpdate",
    "ActionType",
    "ActionUpdate",
    "ActionUpdateConfigType0",
    "ADPClientCredentialsRequest",
    "AgentData",
    "AgentDataOpenaiProvider",
    "AgentPerformance",
    "AppAuthLiteDoc",
    "AppAuthLiteDocPage",
    "AppAuthRead",
    "AppCredsCreate",
    "AppCredsCreateCreds",
    "AppCredsDoc",
    "AppCredsDocCreds",
    "AppCredsLiteDoc",
    "AppCredsLiteDocPage",
    "AppCredsRead",
    "AppCredsUpdate",
    "AppCredsUpdateCreds",
    "AppTypes",
    "Audio",
    "AudioRes",
    "AuthResponse",
    "AutoFileChunkingStrategyParam",
    "BaseModel",
    "BearerAccessToken",
    "BodyAsyncRunActionLoopV1ActionsIdAsyncRunLoopPost",
    "BodyAsyncRunActionLoopV1ActionsIdAsyncRunLoopPostInputItem",
    "BodyAsyncRunActionV1ActionsIdAsyncRunPost",
    "BodyAsyncRunActionV1ActionsIdAsyncRunPostInput",
    "BodyCookieCookiePost",
    "BodyIngestFileAsyncV1IngestionFileAsyncPost",
    "BodyProcessPasswordResetPasswordResetPost",
    "BodyReturnTokenV1AuthTokenPost",
    "BodyReturnUserAndSessionV1AuthPost",
    "BodyRunSlackCmdV1WebhooksSlackCommandPost",
    "BodySignupSignupPost",
    "BodyUploadFileOnAzureV1OpenaiStorageFilesAzurePost",
    "BodyUploadFileOnOpenaiV1OpenaiStorageFilesOpenaiPost",
    "BodyUploadFilesV1FilesPost",
    "BodyUploadFileV1OpenaiStorageFilesPost",
    "BodyUploadPublicFilesV1FilesPublicPost",
    "BodyV2IngestFileIntoCollectionV1VectorstoreV2CollectionsCollectionNameFilesAsyncPost",
    "BulkMemberResponse",
    "CharacterCreate",
    "CharacterDoc",
    "CharacterLiteDoc",
    "CharacterLiteDocPage",
    "CharacterUpdate",
    "ChartData",
    "ChatCompletionAllowedToolChoiceParam",
    "ChatCompletionAllowedToolsParam",
    "ChatCompletionAllowedToolsParamMode",
    "ChatCompletionAllowedToolsParamToolsItem",
    "ChatCompletionAssistantMessageParam",
    "ChatCompletionAudioParam",
    "ChatCompletionAudioParamFormat",
    "ChatCompletionAudioParamVoiceType1",
    "ChatCompletionContentPartImageParam",
    "ChatCompletionContentPartInputAudioParam",
    "ChatCompletionContentPartRefusalParam",
    "ChatCompletionContentPartTextParam",
    "ChatCompletionDeveloperMessageParam",
    "ChatCompletionFunctionMessageParam",
    "ChatCompletionPredictionContentParam",
    "ChatCompletionStreamOptionsParam",
    "ChatCompletionSystemMessageParam",
    "ChatCompletionToolMessageParam",
    "ChatCompletionUserMessageParam",
    "ChatDoc",
    "ChatDocOpenaiProvider",
    "ChatUpdate",
    "CloneActionRequest",
    "CloneActionResult",
    "CloneActionV1ActionsClonePostResponseCloneActionV1ActionsClonePost",
    "CodeInterpreterTool",
    "CodeInterpreterToolParam",
    "CostConfigCreate",
    "CostConfigListResponse",
    "CostConfigResponse",
    "CostConfigType",
    "CostConfigUpdate",
    "CreateChatV1ActionChatsPostOpenaiProvider",
    "CreateVectorStore",
    "CreateVectorStoreFile",
    "CreateVectorStoreFileBatch",
    "CreateVectorStoreFilesV1OpenaiStorageVectorStoreFileBatchesPostOpenaiProvider",
    "CreateVectorStoreFilesV1OpenaiStorageVectorStoreFilesPostOpenaiProvider",
    "CreateVectorStoreMetadataType0",
    "CreateVectorStoreV1OpenaiStorageVectorStoresPostOpenaiProvider",
    "CronSchedule",
    "CronScheduleDayMode",
    "CronScheduleDayOfWeekType0Item",
    "CronScheduleDayOfWeekType1",
    "CronScheduleHourMode",
    "CronScheduleMinuteMode",
    "CronScheduleMonthMode",
    "CronScheduleWeekMode",
    "Custom",
    "DalleOpenaiConfig",
    "DalleOpenaiConfigProvider",
    "DalleOpenaiConfigQuality",
    "DalleOpenaiConfigResponseFormatType0",
    "DalleOpenaiConfigSizeType0",
    "DalleOpenaiConfigStyleType0",
    "DashboardMetric",
    "DashboardRequest",
    "DashboardResponse",
    "DashboardSummary",
    "DatabaseConnection",
    "DatabaseCredsModel",
    "DatabaseCredsModelCredentialsInfoType0",
    "DatabaseEnum",
    "DataFlowType",
    "DatastoreMetaDoc",
    "DbAuthDoc",
    "DbAuthDocAuth",
    "DbAuthLiteDoc",
    "DbAuthLiteDocPage",
    "DeleteCostConfigV1DashboardCostConfigConfigTypeDeleteResponseDeleteCostConfigV1DashboardCostConfigConfigTypeDelete",
    "Deleted",
    "DeleteFileV1OpenaiStorageFilesDeleteOpenaiProvider",
    "DeleteVectorStore",
    "DeleteVectorStoreFile",
    "DeleteVectorStoreFilesV1OpenaiStorageVectorStoreFileBatchesDeleteOpenaiProvider",
    "DeleteVectorStoreV1OpenaiStorageVectorStoresDeleteOpenaiProvider",
    "DepartmentMetrics",
    "DocFindPage",
    "EventResultStatus",
    "EventType",
    "ExpiresAfter",
    "Factor",
    "FactorFactorTypeType0",
    "FactorStatus",
    "File",
    "FileAndProcessingParams",
    "FileCounts",
    "FileDeleted",
    "FileFile",
    "FileListResponseSchema",
    "FileMeta",
    "FileObject",
    "FileObjectPurpose",
    "FileObjectStatus",
    "FileParams",
    "FilesAndProcessingParams",
    "FileSearch",
    "FileSearchRankingOptions",
    "FileSearchRankingOptionsRankerType0",
    "FilterBy",
    "Function",
    "FunctionCall",
    "FunctionDefinition",
    "FunctionDefinitionParametersType0",
    "GetCredsTypesV1AppCredsTypesAppGetResponseGetCredsTypesV1AppCredsTypesAppGet",
    "GetFileInfoV1OpenaiStorageFilesGetOpenaiProvider",
    "GetFileV1OpenaiStorageFilesContentGetOpenaiProvider",
    "GetVectorStoreFilesV1OpenaiStorageVectorStoreFileBatchesGetOpenaiProvider",
    "GetVectorStoreV1OpenaiStorageVectorStoresGetOpenaiProvider",
    "GraphRAGSearchData",
    "GraphRAGSearchDataGraphVisualizationType0",
    "HTTPValidationError",
    "Image",
    "ImageCreateVariation",
    "ImageCreateVariationExtraHeadersType0",
    "ImageCreateVariationExtraQueryType0",
    "ImageCreateVariationResponseFormatType0",
    "ImageCreateVariationSizeType0",
    "ImageEdit",
    "ImageEditExtraHeadersType0",
    "ImageEditExtraQueryType0",
    "ImageEditInput",
    "ImageEditInputSizeType0",
    "ImageEditResponseFormatType0",
    "ImageEditSizeType0",
    "ImagesResponse",
    "ImagesResponseBackgroundType0",
    "ImagesResponseOutputFormatType0",
    "ImagesResponseQualityType0",
    "ImagesResponseSizeType0",
    "ImageURL",
    "ImageURLDetail",
    "ImageVariationInput",
    "ImageVariationInputSizeType0",
    "IngestAsyncResponse",
    "InitializeDefaultConfigsV1DashboardCostConfigInitializePostResponseInitializeDefaultConfigsV1DashboardCostConfigInitializePost",
    "InputAudio",
    "InputAudioFormat",
    "IntervalTriggerModel",
    "JobStatusSchema",
    "JSONSchema",
    "JSONSchemaSchema",
    "KnowledgeBaseCreateRequest",
    "KnowledgeBaseMemberByEmailRequest",
    "KnowledgeBaseMemberCreateRequest",
    "KnowledgeBaseMembersBulkRequest",
    "KnowledgeBaseMembershipResponse",
    "KnowledgeBaseMemberUpdateRequest",
    "KnowledgeBaseResponse",
    "KnowledgeBaseRole",
    "KnowledgeBaseUpdateRequest",
    "LastError",
    "LastErrorCode",
    "ListActionToolsV1ActionsToolsGetLlmType",
    "ListFilesInfoV1OpenaiStorageFilesListGetOpenaiProvider",
    "ListIngestedFilesProxyV1IngestionFilesGetScopeType0",
    "ListIngestedFilesProxyV1IngestionFilesGetSortBy",
    "ListIngestedFilesProxyV1IngestionFilesGetSortOrder",
    "ListMcpsV1McpsGetRequestHeaders",
    "ListMcpsV1McpsGetServerType",
    "ListVectorStoreFilesV1OpenaiStorageVectorStoreFileBatchesListGetFilterType0",
    "ListVectorStoreFilesV1OpenaiStorageVectorStoreFileBatchesListGetOpenaiProvider",
    "ListVectorStoreFilesV1OpenaiStorageVectorStoreFileBatchesListGetOrderType0",
    "ListVectorStoreFilesV1OpenaiStorageVectorStoreFilesListGetFilterType0",
    "ListVectorStoreFilesV1OpenaiStorageVectorStoreFilesListGetOpenaiProvider",
    "ListVectorStoreFilesV1OpenaiStorageVectorStoreFilesListGetOrderType0",
    "ListVectorStoresV1OpenaiStorageVectorStoresListGetOpenaiProvider",
    "ListVectorStoresV1OpenaiStorageVectorStoresListGetOrder",
    "LlmChatOpenaiConfig",
    "LlmChatOpenaiConfigExtraHeadersType0",
    "LlmChatOpenaiConfigExtraQueryType0",
    "LlmChatOpenaiConfigJsonSchemaType0",
    "LlmChatOpenaiConfigLogitBiasType0",
    "LlmChatOpenaiConfigMetadataType0",
    "LlmChatOpenaiConfigModalitiesType0Item",
    "LlmChatOpenaiConfigModelType0",
    "LlmChatOpenaiConfigModelType1",
    "LlmChatOpenaiConfigPromptCacheRetentionType0",
    "LlmChatOpenaiConfigProvider",
    "LlmChatOpenaiConfigReasoningEffortType0",
    "LlmChatOpenaiConfigResponseSchemaType1",
    "LlmChatOpenaiConfigServiceTierType0",
    "LlmChatOpenaiConfigToolChoiceType0",
    "LlmChatOpenaiConfigVerbosityType0",
    "LookerClientCredentialsRequest",
    "MCPCreate",
    "MCPCreateRequestHeadersType0",
    "MCPCreateServerType",
    "MCPDoc",
    "MCPDocRequestHeadersType0",
    "MCPDocServerType",
    "MCPDocsFound",
    "MCPUpdate",
    "MCPUpdateRequestHeadersType0",
    "MCPUpdateServerTypeType0",
    "Message",
    "MessageContentType1Item",
    "Metadata",
    "MetricHistoryResponse",
    "MetricHistoryResponseSummaryStats",
    "MetricType",
    "NetSuiteTBARequest",
    "Node",
    "NodeData",
    "OtherFileChunkingStrategyObject",
    "OutputType",
    "PartitionParametersParams",
    "PartitionParametersParamsHiResModelNameType0",
    "PartitionParametersParamsStrategyType0",
    "Position",
    "PromptUpdate",
    "ReadUrlConfig",
    "Region",
    "ResponseFormatJSONObject",
    "ResponseFormatJSONSchema",
    "ResponseFormatText",
    "RetrieveVectorStore",
    "RetrieveVectorStoreFileBatch",
    "Role",
    "RunActionStreamV1ActionsIdRunStreamPostInput",
    "RunActionV1ActionsIdRunPostInput",
    "RunnableType",
    "RunReactAgentV1AgentsReactPostOpenaiProvider",
    "RunWebhookEventV1WebhooksEventIdPostEvent",
    "ScheduleUpdate",
    "SearchResult",
    "SearchResultMetadataType0",
    "SearchTextParams",
    "SearchTextParamsSafesearch",
    "SearchWebConfig",
    "SearchWebConfigSearchType",
    "Session",
    "ShareDataCreate",
    "ShareDataDoc",
    "ShareDataDocsPage",
    "ShareDataUpdate",
    "SiriAutomationPayload",
    "SourceChunk",
    "SourceChunkMetadata",
    "SpeechReq",
    "SpeechReqModel",
    "SpeechReqResponseFormat",
    "SpeechReqVoice",
    "SpotifyClientCredentialsRequest",
    "StaticFileChunkingStrategy",
    "StaticFileChunkingStrategyObject",
    "StaticFileChunkingStrategyObjectParam",
    "StaticFileChunkingStrategyParam",
    "StructuredOutputSchema",
    "StructuredOutputSchemaProperties",
    "SyncAppAuthV1AppAuthSyncGetResponseSyncAppAuthV1AppAuthSyncGet",
    "TeamCreate",
    "TeamDoc",
    "TeamLiteDoc",
    "TeamLiteDocPage",
    "TeamUpdate",
    "TestDbConnectionV1DbAuthTestConnectionDbAuthIdPostResponseTestDbConnectionV1DbAuthTestConnectionDbAuthIdPost",
    "TextObj",
    "TextObjs",
    "Timelimit",
    "TimeRange",
    "TimeRangeConfig",
    "TimeSeriesDataPoint",
    "TimezoneEnum",
    "ToolResources",
    "ToolResourcesCodeInterpreter",
    "ToolResourcesFileSearch",
    "ToolResourcesFileSearchVectorStore",
    "ToolResourcesFileSearchVectorStoreChunkingStrategyAuto",
    "ToolResourcesFileSearchVectorStoreChunkingStrategyStatic",
    "ToolResourcesFileSearchVectorStoreChunkingStrategyStaticStatic",
    "ToolResourcesFileSearchVectorStoreMetadataType0",
    "TranscriptionReq",
    "TranscriptionReqResponseFormat",
    "TranslationReq",
    "TranslationReqResponseFormat",
    "UpdateVectorStore",
    "UpdateVectorStoreV1OpenaiStorageVectorStoresPutOpenaiProvider",
    "UrlObj",
    "UrlResponse",
    "Usage",
    "UsageInputTokensDetails",
    "UsageOutputTokensDetails",
    "User",
    "UserAppMetadata",
    "UserAttributes",
    "UserIdentity",
    "UserIdentityIdentityData",
    "UserUserMetadata",
    "V2DeleteCollectionV1VectorstoreV2CollectionsCollectionNameDeleteResponseV2DeleteCollectionV1VectorstoreV2CollectionsCollectionNameDelete",
    "ValidationError",
    "ValidationErrorContext",
    "VectorStoreDeleted",
    "VectorStoreFile",
    "VectorStoreFileAttributesType0",
    "VectorStoreFileBatch",
    "VectorStoreFileBatchStatus",
    "VectorStoreFileDeleted",
    "VectorStoreFileStatus",
    "Webhook",
    "WebhookCreate",
    "WebhookDoc",
    "WebhookUpdate",
    "WebSearchOptions",
    "WebSearchOptionsSearchContextSize",
    "WebSearchOptionsUserLocation",
    "WebSearchOptionsUserLocationApproximate",
    "WorkflowPerformance",
    "ZendeskWebhookV1WebhooksZendeskPostEvent",
)



from __future__ import annotations

from collections.abc import Mapping
from typing import Any, TypeVar

from attrs import define as _attrs_define
from attrs import field as _attrs_field

T = TypeVar("T", bound="WorkflowPerformance")


@_attrs_define
class WorkflowPerformance:
    """Workflow performance metrics

    Attributes:
        workflow_id (str):
        workflow_name (str):
        percentage (float):
        change (float):
        rank (int):
    """

    workflow_id: str
    workflow_name: str
    percentage: float
    change: float
    rank: int
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        workflow_id = self.workflow_id

        workflow_name = self.workflow_name

        percentage = self.percentage

        change = self.change

        rank = self.rank

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "workflow_id": workflow_id,
                "workflow_name": workflow_name,
                "percentage": percentage,
                "change": change,
                "rank": rank,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        d = dict(src_dict)
        workflow_id = d.pop("workflow_id")

        workflow_name = d.pop("workflow_name")

        percentage = d.pop("percentage")

        change = d.pop("change")

        rank = d.pop("rank")

        workflow_performance = cls(
            workflow_id=workflow_id,
            workflow_name=workflow_name,
            percentage=percentage,
            change=change,
            rank=rank,
        )

        workflow_performance.additional_properties = d
        return workflow_performance

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from __future__ import annotations

import datetime
from collections.abc import Mapping
from typing import TYPE_CHECKING, Any, TypeVar

from attrs import define as _attrs_define
from attrs import field as _attrs_field
from dateutil.parser import isoparse

if TYPE_CHECKING:
    from ..models.agent_performance import AgentPerformance
    from ..models.chart_data import ChartData
    from ..models.dashboard_summary import DashboardSummary
    from ..models.department_metrics import DepartmentMetrics
    from ..models.workflow_performance import WorkflowPerformance


T = TypeVar("T", bound="DashboardResponse")


@_attrs_define
class DashboardResponse:
    """Complete dashboard response

    Attributes:
        summary (DashboardSummary): Main dashboard summary
        time_saved_chart (ChartData): Chart data structure
        dollars_saved_chart (ChartData): Chart data structure
        roi_chart (ChartData): Chart data structure
        revenue_chart (ChartData): Chart data structure
        top_agents (list[AgentPerformance]):
        top_workflows (list[WorkflowPerformance]):
        department_metrics (list[DepartmentMetrics]):
        generated_at (datetime.datetime):
    """

    summary: DashboardSummary
    time_saved_chart: ChartData
    dollars_saved_chart: ChartData
    roi_chart: ChartData
    revenue_chart: ChartData
    top_agents: list[AgentPerformance]
    top_workflows: list[WorkflowPerformance]
    department_metrics: list[DepartmentMetrics]
    generated_at: datetime.datetime
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        summary = self.summary.to_dict()

        time_saved_chart = self.time_saved_chart.to_dict()

        dollars_saved_chart = self.dollars_saved_chart.to_dict()

        roi_chart = self.roi_chart.to_dict()

        revenue_chart = self.revenue_chart.to_dict()

        top_agents = []
        for top_agents_item_data in self.top_agents:
            top_agents_item = top_agents_item_data.to_dict()
            top_agents.append(top_agents_item)

        top_workflows = []
        for top_workflows_item_data in self.top_workflows:
            top_workflows_item = top_workflows_item_data.to_dict()
            top_workflows.append(top_workflows_item)

        department_metrics = []
        for department_metrics_item_data in self.department_metrics:
            department_metrics_item = department_metrics_item_data.to_dict()
            department_metrics.append(department_metrics_item)

        generated_at = self.generated_at.isoformat()

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "summary": summary,
                "time_saved_chart": time_saved_chart,
                "dollars_saved_chart": dollars_saved_chart,
                "roi_chart": roi_chart,
                "revenue_chart": revenue_chart,
                "top_agents": top_agents,
                "top_workflows": top_workflows,
                "department_metrics": department_metrics,
                "generated_at": generated_at,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        from ..models.agent_performance import AgentPerformance
        from ..models.chart_data import ChartData
        from ..models.dashboard_summary import DashboardSummary
        from ..models.department_metrics import DepartmentMetrics
        from ..models.workflow_performance import WorkflowPerformance

        d = dict(src_dict)
        summary = DashboardSummary.from_dict(d.pop("summary"))

        time_saved_chart = ChartData.from_dict(d.pop("time_saved_chart"))

        dollars_saved_chart = ChartData.from_dict(d.pop("dollars_saved_chart"))

        roi_chart = ChartData.from_dict(d.pop("roi_chart"))

        revenue_chart = ChartData.from_dict(d.pop("revenue_chart"))

        top_agents = []
        _top_agents = d.pop("top_agents")
        for top_agents_item_data in _top_agents:
            top_agents_item = AgentPerformance.from_dict(top_agents_item_data)

            top_agents.append(top_agents_item)

        top_workflows = []
        _top_workflows = d.pop("top_workflows")
        for top_workflows_item_data in _top_workflows:
            top_workflows_item = WorkflowPerformance.from_dict(top_workflows_item_data)

            top_workflows.append(top_workflows_item)

        department_metrics = []
        _department_metrics = d.pop("department_metrics")
        for department_metrics_item_data in _department_metrics:
            department_metrics_item = DepartmentMetrics.from_dict(department_metrics_item_data)

            department_metrics.append(department_metrics_item)

        generated_at = isoparse(d.pop("generated_at"))

        dashboard_response = cls(
            summary=summary,
            time_saved_chart=time_saved_chart,
            dollars_saved_chart=dollars_saved_chart,
            roi_chart=roi_chart,
            revenue_chart=revenue_chart,
            top_agents=top_agents,
            top_workflows=top_workflows,
            department_metrics=department_metrics,
            generated_at=generated_at,
        )

        dashboard_response.additional_properties = d
        return dashboard_response

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from enum import Enum


class ChatCompletionAudioParamVoiceType1(str, Enum):
    ALLOY = "alloy"
    ASH = "ash"
    BALLAD = "ballad"
    CEDAR = "cedar"
    CORAL = "coral"
    ECHO = "echo"
    MARIN = "marin"
    SAGE = "sage"
    SHIMMER = "shimmer"
    VERSE = "verse"

    def __str__(self) -> str:
        return str(self.value)



from __future__ import annotations

from collections.abc import Mapping
from typing import Any, TypeVar, cast

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..models.image_variation_input_size_type_0 import ImageVariationInputSizeType0
from ..types import UNSET, Unset

T = TypeVar("T", bound="ImageVariationInput")


@_attrs_define
class ImageVariationInput:
    """
    Attributes:
        image (None | str | Unset):
        size (ImageVariationInputSizeType0 | None | Unset):
    """

    image: None | str | Unset = UNSET
    size: ImageVariationInputSizeType0 | None | Unset = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        image: None | str | Unset
        if isinstance(self.image, Unset):
            image = UNSET
        else:
            image = self.image

        size: None | str | Unset
        if isinstance(self.size, Unset):
            size = UNSET
        elif isinstance(self.size, ImageVariationInputSizeType0):
            size = self.size.value
        else:
            size = self.size

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if image is not UNSET:
            field_dict["image"] = image
        if size is not UNSET:
            field_dict["size"] = size

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        d = dict(src_dict)

        def _parse_image(data: object) -> None | str | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(None | str | Unset, data)

        image = _parse_image(d.pop("image", UNSET))

        def _parse_size(data: object) -> ImageVariationInputSizeType0 | None | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, str):
                    raise TypeError()
                size_type_0 = ImageVariationInputSizeType0(data)

                return size_type_0
            except (TypeError, ValueError, AttributeError, KeyError):
                pass
            return cast(ImageVariationInputSizeType0 | None | Unset, data)

        size = _parse_size(d.pop("size", UNSET))

        image_variation_input = cls(
            image=image,
            size=size,
        )

        image_variation_input.additional_properties = d
        return image_variation_input

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from http import HTTPStatus
from typing import Any

import httpx

from ... import errors
from ...client import AuthenticatedClient, Client
from ...models.http_validation_error import HTTPValidationError
from ...models.time_range import TimeRange
from ...types import UNSET, Response, Unset


def _get_kwargs(
    *,
    time_range: TimeRange | Unset = UNSET,
    force_refresh: bool | Unset = False,
) -> dict[str, Any]:

    params: dict[str, Any] = {}

    json_time_range: str | Unset = UNSET
    if not isinstance(time_range, Unset):
        json_time_range = time_range.value

    params["time_range"] = json_time_range

    params["force_refresh"] = force_refresh

    params = {k: v for k, v in params.items() if v is not UNSET and v is not None}

    _kwargs: dict[str, Any] = {
        "method": "get",
        "url": "/v1/dashboard/summary",
        "params": params,
    }

    return _kwargs


def _parse_response(
    *, client: AuthenticatedClient | Client, response: httpx.Response
) -> Any | HTTPValidationError | None:
    if response.status_code == 200:
        response_200 = response.json()
        return response_200

    if response.status_code == 422:
        response_422 = HTTPValidationError.from_dict(response.json())

        return response_422

    if client.raise_on_unexpected_status:
        raise errors.UnexpectedStatus(response.status_code, response.content)
    else:
        return None


def _build_response(
    *, client: AuthenticatedClient | Client, response: httpx.Response
) -> Response[Any | HTTPValidationError]:
    return Response(
        status_code=HTTPStatus(response.status_code),
        content=response.content,
        headers=response.headers,
        parsed=_parse_response(client=client, response=response),
    )


def sync_detailed(
    *,
    client: AuthenticatedClient,
    time_range: TimeRange | Unset = UNSET,
    force_refresh: bool | Unset = False,
) -> Response[Any | HTTPValidationError]:
    """Get Dashboard Summary

     Get just the summary metrics without charts and detailed data.

    Useful for quick overview widgets or mobile views.
    Data is served from MongoDB cache for faster response times.

    Args:
        time_range (TimeRange | Unset):
        force_refresh (bool | Unset): Force refresh cache Default: False.

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[Any | HTTPValidationError]
    """

    kwargs = _get_kwargs(
        time_range=time_range,
        force_refresh=force_refresh,
    )

    response = client.get_httpx_client().request(
        **kwargs,
    )

    return _build_response(client=client, response=response)


def sync(
    *,
    client: AuthenticatedClient,
    time_range: TimeRange | Unset = UNSET,
    force_refresh: bool | Unset = False,
) -> Any | HTTPValidationError | None:
    """Get Dashboard Summary

     Get just the summary metrics without charts and detailed data.

    Useful for quick overview widgets or mobile views.
    Data is served from MongoDB cache for faster response times.

    Args:
        time_range (TimeRange | Unset):
        force_refresh (bool | Unset): Force refresh cache Default: False.

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Any | HTTPValidationError
    """

    return sync_detailed(
        client=client,
        time_range=time_range,
        force_refresh=force_refresh,
    ).parsed


async def asyncio_detailed(
    *,
    client: AuthenticatedClient,
    time_range: TimeRange | Unset = UNSET,
    force_refresh: bool | Unset = False,
) -> Response[Any | HTTPValidationError]:
    """Get Dashboard Summary

     Get just the summary metrics without charts and detailed data.

    Useful for quick overview widgets or mobile views.
    Data is served from MongoDB cache for faster response times.

    Args:
        time_range (TimeRange | Unset):
        force_refresh (bool | Unset): Force refresh cache Default: False.

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[Any | HTTPValidationError]
    """

    kwargs = _get_kwargs(
        time_range=time_range,
        force_refresh=force_refresh,
    )

    response = await client.get_async_httpx_client().request(**kwargs)

    return _build_response(client=client, response=response)


async def asyncio(
    *,
    client: AuthenticatedClient,
    time_range: TimeRange | Unset = UNSET,
    force_refresh: bool | Unset = False,
) -> Any | HTTPValidationError | None:
    """Get Dashboard Summary

     Get just the summary metrics without charts and detailed data.

    Useful for quick overview widgets or mobile views.
    Data is served from MongoDB cache for faster response times.

    Args:
        time_range (TimeRange | Unset):
        force_refresh (bool | Unset): Force refresh cache Default: False.

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Any | HTTPValidationError
    """

    return (
        await asyncio_detailed(
            client=client,
            time_range=time_range,
            force_refresh=force_refresh,
        )
    ).parsed



from __future__ import annotations

from collections.abc import Mapping
from typing import Any, TypeVar

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

T = TypeVar("T", bound="ChatCompletionStreamOptionsParam")


@_attrs_define
class ChatCompletionStreamOptionsParam:
    """Options for streaming response. Only set this when you set `stream: true`.

    Attributes:
        include_obfuscation (bool | Unset):
        include_usage (bool | Unset):
    """

    include_obfuscation: bool | Unset = UNSET
    include_usage: bool | Unset = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        include_obfuscation = self.include_obfuscation

        include_usage = self.include_usage

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if include_obfuscation is not UNSET:
            field_dict["include_obfuscation"] = include_obfuscation
        if include_usage is not UNSET:
            field_dict["include_usage"] = include_usage

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        d = dict(src_dict)
        include_obfuscation = d.pop("include_obfuscation", UNSET)

        include_usage = d.pop("include_usage", UNSET)

        chat_completion_stream_options_param = cls(
            include_obfuscation=include_obfuscation,
            include_usage=include_usage,
        )

        chat_completion_stream_options_param.additional_properties = d
        return chat_completion_stream_options_param

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from __future__ import annotations

import datetime
from collections.abc import Mapping
from typing import Any, Literal, TypeVar, cast

from attrs import define as _attrs_define
from attrs import field as _attrs_field
from dateutil.parser import isoparse

from ..models.cron_schedule_day_mode import CronScheduleDayMode
from ..models.cron_schedule_day_of_week_type_0_item import CronScheduleDayOfWeekType0Item
from ..models.cron_schedule_day_of_week_type_1 import CronScheduleDayOfWeekType1
from ..models.cron_schedule_hour_mode import CronScheduleHourMode
from ..models.cron_schedule_minute_mode import CronScheduleMinuteMode
from ..models.cron_schedule_month_mode import CronScheduleMonthMode
from ..models.cron_schedule_week_mode import CronScheduleWeekMode
from ..models.timezone_enum import TimezoneEnum
from ..types import UNSET, Unset

T = TypeVar("T", bound="CronSchedule")


@_attrs_define
class CronSchedule:
    """
    Attributes:
        trigger (Literal['cron'] | Unset):  Default: 'cron'.
        minute (int | None | Unset): minute (0-59)
        hour (int | None | Unset): hour (0-23)
        day (int | None | Unset): day of month (1-31)
        day_of_week (CronScheduleDayOfWeekType1 | list[CronScheduleDayOfWeekType0Item] | None | Unset): number mapped
            name of weekday (0-6 or mon,tue,wed,thu,fri,sat,sun)
        week (int | None | Unset): ISO week (1-53)
        month (int | None | Unset): month (1-12)
        year (int | None | Unset): 4-digit year
        start_date (datetime.datetime | None | Unset): earliest possible date/time to trigger on (inclusive)
        end_date (datetime.datetime | None | Unset): latest possible date/time to trigger on (inclusive)
        timezone (None | TimezoneEnum | Unset): time zone to use for the date/time calculations (defaults to scheduler
            timezone)
        jitter (int | None | Unset): delay the job execution by ``jitter`` seconds at most
        minute_mode (CronScheduleMinuteMode | Unset):  Default: CronScheduleMinuteMode.AT.
        hour_mode (CronScheduleHourMode | Unset):  Default: CronScheduleHourMode.AT.
        day_mode (CronScheduleDayMode | Unset):  Default: CronScheduleDayMode.AT.
        week_mode (CronScheduleWeekMode | Unset):  Default: CronScheduleWeekMode.AT.
        month_mode (CronScheduleMonthMode | Unset):  Default: CronScheduleMonthMode.AT.
    """

    trigger: Literal["cron"] | Unset = "cron"
    minute: int | None | Unset = UNSET
    hour: int | None | Unset = UNSET
    day: int | None | Unset = UNSET
    day_of_week: CronScheduleDayOfWeekType1 | list[CronScheduleDayOfWeekType0Item] | None | Unset = UNSET
    week: int | None | Unset = UNSET
    month: int | None | Unset = UNSET
    year: int | None | Unset = UNSET
    start_date: datetime.datetime | None | Unset = UNSET
    end_date: datetime.datetime | None | Unset = UNSET
    timezone: None | TimezoneEnum | Unset = UNSET
    jitter: int | None | Unset = UNSET
    minute_mode: CronScheduleMinuteMode | Unset = CronScheduleMinuteMode.AT
    hour_mode: CronScheduleHourMode | Unset = CronScheduleHourMode.AT
    day_mode: CronScheduleDayMode | Unset = CronScheduleDayMode.AT
    week_mode: CronScheduleWeekMode | Unset = CronScheduleWeekMode.AT
    month_mode: CronScheduleMonthMode | Unset = CronScheduleMonthMode.AT
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        trigger = self.trigger

        minute: int | None | Unset
        if isinstance(self.minute, Unset):
            minute = UNSET
        else:
            minute = self.minute

        hour: int | None | Unset
        if isinstance(self.hour, Unset):
            hour = UNSET
        else:
            hour = self.hour

        day: int | None | Unset
        if isinstance(self.day, Unset):
            day = UNSET
        else:
            day = self.day

        day_of_week: list[str] | None | str | Unset
        if isinstance(self.day_of_week, Unset):
            day_of_week = UNSET
        elif isinstance(self.day_of_week, list):
            day_of_week = []
            for day_of_week_type_0_item_data in self.day_of_week:
                day_of_week_type_0_item = day_of_week_type_0_item_data.value
                day_of_week.append(day_of_week_type_0_item)

        elif isinstance(self.day_of_week, CronScheduleDayOfWeekType1):
            day_of_week = self.day_of_week.value
        else:
            day_of_week = self.day_of_week

        week: int | None | Unset
        if isinstance(self.week, Unset):
            week = UNSET
        else:
            week = self.week

        month: int | None | Unset
        if isinstance(self.month, Unset):
            month = UNSET
        else:
            month = self.month

        year: int | None | Unset
        if isinstance(self.year, Unset):
            year = UNSET
        else:
            year = self.year

        start_date: None | str | Unset
        if isinstance(self.start_date, Unset):
            start_date = UNSET
        elif isinstance(self.start_date, datetime.datetime):
            start_date = self.start_date.isoformat()
        else:
            start_date = self.start_date

        end_date: None | str | Unset
        if isinstance(self.end_date, Unset):
            end_date = UNSET
        elif isinstance(self.end_date, datetime.datetime):
            end_date = self.end_date.isoformat()
        else:
            end_date = self.end_date

        timezone: None | str | Unset
        if isinstance(self.timezone, Unset):
            timezone = UNSET
        elif isinstance(self.timezone, TimezoneEnum):
            timezone = self.timezone.value
        else:
            timezone = self.timezone

        jitter: int | None | Unset
        if isinstance(self.jitter, Unset):
            jitter = UNSET
        else:
            jitter = self.jitter

        minute_mode: str | Unset = UNSET
        if not isinstance(self.minute_mode, Unset):
            minute_mode = self.minute_mode.value

        hour_mode: str | Unset = UNSET
        if not isinstance(self.hour_mode, Unset):
            hour_mode = self.hour_mode.value

        day_mode: str | Unset = UNSET
        if not isinstance(self.day_mode, Unset):
            day_mode = self.day_mode.value

        week_mode: str | Unset = UNSET
        if not isinstance(self.week_mode, Unset):
            week_mode = self.week_mode.value

        month_mode: str | Unset = UNSET
        if not isinstance(self.month_mode, Unset):
            month_mode = self.month_mode.value

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if trigger is not UNSET:
            field_dict["trigger"] = trigger
        if minute is not UNSET:
            field_dict["minute"] = minute
        if hour is not UNSET:
            field_dict["hour"] = hour
        if day is not UNSET:
            field_dict["day"] = day
        if day_of_week is not UNSET:
            field_dict["day_of_week"] = day_of_week
        if week is not UNSET:
            field_dict["week"] = week
        if month is not UNSET:
            field_dict["month"] = month
        if year is not UNSET:
            field_dict["year"] = year
        if start_date is not UNSET:
            field_dict["start_date"] = start_date
        if end_date is not UNSET:
            field_dict["end_date"] = end_date
        if timezone is not UNSET:
            field_dict["timezone"] = timezone
        if jitter is not UNSET:
            field_dict["jitter"] = jitter
        if minute_mode is not UNSET:
            field_dict["minute_mode"] = minute_mode
        if hour_mode is not UNSET:
            field_dict["hour_mode"] = hour_mode
        if day_mode is not UNSET:
            field_dict["day_mode"] = day_mode
        if week_mode is not UNSET:
            field_dict["week_mode"] = week_mode
        if month_mode is not UNSET:
            field_dict["month_mode"] = month_mode

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        d = dict(src_dict)
        trigger = cast(Literal["cron"] | Unset, d.pop("trigger", UNSET))
        if trigger != "cron" and not isinstance(trigger, Unset):
            raise ValueError(f"trigger must match const 'cron', got '{trigger}'")

        def _parse_minute(data: object) -> int | None | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(int | None | Unset, data)

        minute = _parse_minute(d.pop("minute", UNSET))

        def _parse_hour(data: object) -> int | None | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(int | None | Unset, data)

        hour = _parse_hour(d.pop("hour", UNSET))

        def _parse_day(data: object) -> int | None | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(int | None | Unset, data)

        day = _parse_day(d.pop("day", UNSET))

        def _parse_day_of_week(
            data: object,
        ) -> CronScheduleDayOfWeekType1 | list[CronScheduleDayOfWeekType0Item] | None | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, list):
                    raise TypeError()
                day_of_week_type_0 = []
                _day_of_week_type_0 = data
                for day_of_week_type_0_item_data in _day_of_week_type_0:
                    day_of_week_type_0_item = CronScheduleDayOfWeekType0Item(day_of_week_type_0_item_data)

                    day_of_week_type_0.append(day_of_week_type_0_item)

                return day_of_week_type_0
            except (TypeError, ValueError, AttributeError, KeyError):
                pass
            try:
                if not isinstance(data, str):
                    raise TypeError()
                day_of_week_type_1 = CronScheduleDayOfWeekType1(data)

                return day_of_week_type_1
            except (TypeError, ValueError, AttributeError, KeyError):
                pass
            return cast(CronScheduleDayOfWeekType1 | list[CronScheduleDayOfWeekType0Item] | None | Unset, data)

        day_of_week = _parse_day_of_week(d.pop("day_of_week", UNSET))

        def _parse_week(data: object) -> int | None | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(int | None | Unset, data)

        week = _parse_week(d.pop("week", UNSET))

        def _parse_month(data: object) -> int | None | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(int | None | Unset, data)

        month = _parse_month(d.pop("month", UNSET))

        def _parse_year(data: object) -> int | None | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(int | None | Unset, data)

        year = _parse_year(d.pop("year", UNSET))

        def _parse_start_date(data: object) -> datetime.datetime | None | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, str):
                    raise TypeError()
                start_date_type_0 = isoparse(data)

                return start_date_type_0
            except (TypeError, ValueError, AttributeError, KeyError):
                pass
            return cast(datetime.datetime | None | Unset, data)

        start_date = _parse_start_date(d.pop("start_date", UNSET))

        def _parse_end_date(data: object) -> datetime.datetime | None | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, str):
                    raise TypeError()
                end_date_type_0 = isoparse(data)

                return end_date_type_0
            except (TypeError, ValueError, AttributeError, KeyError):
                pass
            return cast(datetime.datetime | None | Unset, data)

        end_date = _parse_end_date(d.pop("end_date", UNSET))

        def _parse_timezone(data: object) -> None | TimezoneEnum | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, str):
                    raise TypeError()
                timezone_type_0 = TimezoneEnum(data)

                return timezone_type_0
            except (TypeError, ValueError, AttributeError, KeyError):
                pass
            return cast(None | TimezoneEnum | Unset, data)

        timezone = _parse_timezone(d.pop("timezone", UNSET))

        def _parse_jitter(data: object) -> int | None | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(int | None | Unset, data)

        jitter = _parse_jitter(d.pop("jitter", UNSET))

        _minute_mode = d.pop("minute_mode", UNSET)
        minute_mode: CronScheduleMinuteMode | Unset
        if isinstance(_minute_mode, Unset):
            minute_mode = UNSET
        else:
            minute_mode = CronScheduleMinuteMode(_minute_mode)

        _hour_mode = d.pop("hour_mode", UNSET)
        hour_mode: CronScheduleHourMode | Unset
        if isinstance(_hour_mode, Unset):
            hour_mode = UNSET
        else:
            hour_mode = CronScheduleHourMode(_hour_mode)

        _day_mode = d.pop("day_mode", UNSET)
        day_mode: CronScheduleDayMode | Unset
        if isinstance(_day_mode, Unset):
            day_mode = UNSET
        else:
            day_mode = CronScheduleDayMode(_day_mode)

        _week_mode = d.pop("week_mode", UNSET)
        week_mode: CronScheduleWeekMode | Unset
        if isinstance(_week_mode, Unset):
            week_mode = UNSET
        else:
            week_mode = CronScheduleWeekMode(_week_mode)

        _month_mode = d.pop("month_mode", UNSET)
        month_mode: CronScheduleMonthMode | Unset
        if isinstance(_month_mode, Unset):
            month_mode = UNSET
        else:
            month_mode = CronScheduleMonthMode(_month_mode)

        cron_schedule = cls(
            trigger=trigger,
            minute=minute,
            hour=hour,
            day=day,
            day_of_week=day_of_week,
            week=week,
            month=month,
            year=year,
            start_date=start_date,
            end_date=end_date,
            timezone=timezone,
            jitter=jitter,
            minute_mode=minute_mode,
            hour_mode=hour_mode,
            day_mode=day_mode,
            week_mode=week_mode,
            month_mode=month_mode,
        )

        cron_schedule.additional_properties = d
        return cron_schedule

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from __future__ import annotations

from collections.abc import Mapping
from typing import TYPE_CHECKING, Any, TypeVar, cast

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.source_chunk_metadata import SourceChunkMetadata


T = TypeVar("T", bound="SourceChunk")


@_attrs_define
class SourceChunk:
    """
    Attributes:
        chunk_id (str):
        source (str):
        text_preview (str):
        score (float):
        document_id (None | str | Unset):
        s3_url (None | str | Unset):
        metadata (SourceChunkMetadata | Unset):
    """

    chunk_id: str
    source: str
    text_preview: str
    score: float
    document_id: None | str | Unset = UNSET
    s3_url: None | str | Unset = UNSET
    metadata: SourceChunkMetadata | Unset = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        chunk_id = self.chunk_id

        source = self.source

        text_preview = self.text_preview

        score = self.score

        document_id: None | str | Unset
        if isinstance(self.document_id, Unset):
            document_id = UNSET
        else:
            document_id = self.document_id

        s3_url: None | str | Unset
        if isinstance(self.s3_url, Unset):
            s3_url = UNSET
        else:
            s3_url = self.s3_url

        metadata: dict[str, Any] | Unset = UNSET
        if not isinstance(self.metadata, Unset):
            metadata = self.metadata.to_dict()

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "chunk_id": chunk_id,
                "source": source,
                "text_preview": text_preview,
                "score": score,
            }
        )
        if document_id is not UNSET:
            field_dict["document_id"] = document_id
        if s3_url is not UNSET:
            field_dict["s3_url"] = s3_url
        if metadata is not UNSET:
            field_dict["metadata"] = metadata

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        from ..models.source_chunk_metadata import SourceChunkMetadata

        d = dict(src_dict)
        chunk_id = d.pop("chunk_id")

        source = d.pop("source")

        text_preview = d.pop("text_preview")

        score = d.pop("score")

        def _parse_document_id(data: object) -> None | str | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(None | str | Unset, data)

        document_id = _parse_document_id(d.pop("document_id", UNSET))

        def _parse_s3_url(data: object) -> None | str | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(None | str | Unset, data)

        s3_url = _parse_s3_url(d.pop("s3_url", UNSET))

        _metadata = d.pop("metadata", UNSET)
        metadata: SourceChunkMetadata | Unset
        if isinstance(_metadata, Unset):
            metadata = UNSET
        else:
            metadata = SourceChunkMetadata.from_dict(_metadata)

        source_chunk = cls(
            chunk_id=chunk_id,
            source=source,
            text_preview=text_preview,
            score=score,
            document_id=document_id,
            s3_url=s3_url,
            metadata=metadata,
        )

        source_chunk.additional_properties = d
        return source_chunk

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from __future__ import annotations

import datetime
from collections.abc import Mapping
from typing import TYPE_CHECKING, Any, TypeVar, cast

from attrs import define as _attrs_define
from attrs import field as _attrs_field
from dateutil.parser import isoparse

from ..models.event_result_status import EventResultStatus
from ..models.event_type import EventType
from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.action_doc import ActionDoc
    from ..models.text_obj import TextObj


T = TypeVar("T", bound="ActionResultDoc")


@_attrs_define
class ActionResultDoc:
    """
    Attributes:
        type_ (EventType):
        status (EventResultStatus):
        result (ActionDoc):
        user_id (str):
        field_id (str):
        updated_at (datetime.datetime | Unset):
        name (str | Unset):  Default: ''.
        error_message (None | TextObj | Unset):
        is_saved (bool | Unset):  Default: False.
        created_at (datetime.datetime | Unset):  Default: isoparse('2026-03-12T15:03:48.682683').
        file_url (None | str | Unset):
    """

    type_: EventType
    status: EventResultStatus
    result: ActionDoc
    user_id: str
    field_id: str
    updated_at: datetime.datetime | Unset = UNSET
    name: str | Unset = ""
    error_message: None | TextObj | Unset = UNSET
    is_saved: bool | Unset = False
    created_at: datetime.datetime | Unset = isoparse("2026-03-12T15:03:48.682683")
    file_url: None | str | Unset = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        from ..models.text_obj import TextObj

        type_ = self.type_.value

        status = self.status.value

        result = self.result.to_dict()

        user_id = self.user_id

        field_id = self.field_id

        updated_at: str | Unset = UNSET
        if not isinstance(self.updated_at, Unset):
            updated_at = self.updated_at.isoformat()

        name = self.name

        error_message: dict[str, Any] | None | Unset
        if isinstance(self.error_message, Unset):
            error_message = UNSET
        elif isinstance(self.error_message, TextObj):
            error_message = self.error_message.to_dict()
        else:
            error_message = self.error_message

        is_saved = self.is_saved

        created_at: str | Unset = UNSET
        if not isinstance(self.created_at, Unset):
            created_at = self.created_at.isoformat()

        file_url: None | str | Unset
        if isinstance(self.file_url, Unset):
            file_url = UNSET
        else:
            file_url = self.file_url

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "type": type_,
                "status": status,
                "result": result,
                "user_id": user_id,
                "_id": field_id,
            }
        )
        if updated_at is not UNSET:
            field_dict["updated_at"] = updated_at
        if name is not UNSET:
            field_dict["name"] = name
        if error_message is not UNSET:
            field_dict["error_message"] = error_message
        if is_saved is not UNSET:
            field_dict["is_saved"] = is_saved
        if created_at is not UNSET:
            field_dict["created_at"] = created_at
        if file_url is not UNSET:
            field_dict["file_url"] = file_url

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        from ..models.action_doc import ActionDoc
        from ..models.text_obj import TextObj

        d = dict(src_dict)
        type_ = EventType(d.pop("type"))

        status = EventResultStatus(d.pop("status"))

        result = ActionDoc.from_dict(d.pop("result"))

        user_id = d.pop("user_id")

        field_id = d.pop("_id")

        _updated_at = d.pop("updated_at", UNSET)
        updated_at: datetime.datetime | Unset
        if isinstance(_updated_at, Unset):
            updated_at = UNSET
        else:
            updated_at = isoparse(_updated_at)

        name = d.pop("name", UNSET)

        def _parse_error_message(data: object) -> None | TextObj | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                error_message_type_0 = TextObj.from_dict(data)

                return error_message_type_0
            except (TypeError, ValueError, AttributeError, KeyError):
                pass
            return cast(None | TextObj | Unset, data)

        error_message = _parse_error_message(d.pop("error_message", UNSET))

        is_saved = d.pop("is_saved", UNSET)

        _created_at = d.pop("created_at", UNSET)
        created_at: datetime.datetime | Unset
        if isinstance(_created_at, Unset):
            created_at = UNSET
        else:
            created_at = isoparse(_created_at)

        def _parse_file_url(data: object) -> None | str | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(None | str | Unset, data)

        file_url = _parse_file_url(d.pop("file_url", UNSET))

        action_result_doc = cls(
            type_=type_,
            status=status,
            result=result,
            user_id=user_id,
            field_id=field_id,
            updated_at=updated_at,
            name=name,
            error_message=error_message,
            is_saved=is_saved,
            created_at=created_at,
            file_url=file_url,
        )

        action_result_doc.additional_properties = d
        return action_result_doc

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from __future__ import annotations

from collections.abc import Mapping
from typing import TYPE_CHECKING, Any, Literal, TypeVar, cast

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..models.vector_store_file_status import VectorStoreFileStatus
from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.last_error import LastError
    from ..models.other_file_chunking_strategy_object import OtherFileChunkingStrategyObject
    from ..models.static_file_chunking_strategy_object import StaticFileChunkingStrategyObject
    from ..models.vector_store_file_attributes_type_0 import VectorStoreFileAttributesType0


T = TypeVar("T", bound="VectorStoreFile")


@_attrs_define
class VectorStoreFile:
    """A list of files attached to a vector store.

    Attributes:
        id (str):
        created_at (int):
        object_ (Literal['vector_store.file']):
        status (VectorStoreFileStatus):
        usage_bytes (int):
        vector_store_id (str):
        last_error (LastError | None | Unset):
        attributes (None | Unset | VectorStoreFileAttributesType0):
        chunking_strategy (None | OtherFileChunkingStrategyObject | StaticFileChunkingStrategyObject | Unset):
    """

    id: str
    created_at: int
    object_: Literal["vector_store.file"]
    status: VectorStoreFileStatus
    usage_bytes: int
    vector_store_id: str
    last_error: LastError | None | Unset = UNSET
    attributes: None | Unset | VectorStoreFileAttributesType0 = UNSET
    chunking_strategy: None | OtherFileChunkingStrategyObject | StaticFileChunkingStrategyObject | Unset = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        from ..models.last_error import LastError
        from ..models.other_file_chunking_strategy_object import OtherFileChunkingStrategyObject
        from ..models.static_file_chunking_strategy_object import StaticFileChunkingStrategyObject
        from ..models.vector_store_file_attributes_type_0 import VectorStoreFileAttributesType0

        id = self.id

        created_at = self.created_at

        object_ = self.object_

        status = self.status.value

        usage_bytes = self.usage_bytes

        vector_store_id = self.vector_store_id

        last_error: dict[str, Any] | None | Unset
        if isinstance(self.last_error, Unset):
            last_error = UNSET
        elif isinstance(self.last_error, LastError):
            last_error = self.last_error.to_dict()
        else:
            last_error = self.last_error

        attributes: dict[str, Any] | None | Unset
        if isinstance(self.attributes, Unset):
            attributes = UNSET
        elif isinstance(self.attributes, VectorStoreFileAttributesType0):
            attributes = self.attributes.to_dict()
        else:
            attributes = self.attributes

        chunking_strategy: dict[str, Any] | None | Unset
        if isinstance(self.chunking_strategy, Unset):
            chunking_strategy = UNSET
        elif isinstance(self.chunking_strategy, StaticFileChunkingStrategyObject):
            chunking_strategy = self.chunking_strategy.to_dict()
        elif isinstance(self.chunking_strategy, OtherFileChunkingStrategyObject):
            chunking_strategy = self.chunking_strategy.to_dict()
        else:
            chunking_strategy = self.chunking_strategy

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "id": id,
                "created_at": created_at,
                "object": object_,
                "status": status,
                "usage_bytes": usage_bytes,
                "vector_store_id": vector_store_id,
            }
        )
        if last_error is not UNSET:
            field_dict["last_error"] = last_error
        if attributes is not UNSET:
            field_dict["attributes"] = attributes
        if chunking_strategy is not UNSET:
            field_dict["chunking_strategy"] = chunking_strategy

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        from ..models.last_error import LastError
        from ..models.other_file_chunking_strategy_object import OtherFileChunkingStrategyObject
        from ..models.static_file_chunking_strategy_object import StaticFileChunkingStrategyObject
        from ..models.vector_store_file_attributes_type_0 import VectorStoreFileAttributesType0

        d = dict(src_dict)
        id = d.pop("id")

        created_at = d.pop("created_at")

        object_ = cast(Literal["vector_store.file"], d.pop("object"))
        if object_ != "vector_store.file":
            raise ValueError(f"object must match const 'vector_store.file', got '{object_}'")

        status = VectorStoreFileStatus(d.pop("status"))

        usage_bytes = d.pop("usage_bytes")

        vector_store_id = d.pop("vector_store_id")

        def _parse_last_error(data: object) -> LastError | None | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                last_error_type_0 = LastError.from_dict(data)

                return last_error_type_0
            except (TypeError, ValueError, AttributeError, KeyError):
                pass
            return cast(LastError | None | Unset, data)

        last_error = _parse_last_error(d.pop("last_error", UNSET))

        def _parse_attributes(data: object) -> None | Unset | VectorStoreFileAttributesType0:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                attributes_type_0 = VectorStoreFileAttributesType0.from_dict(data)

                return attributes_type_0
            except (TypeError, ValueError, AttributeError, KeyError):
                pass
            return cast(None | Unset | VectorStoreFileAttributesType0, data)

        attributes = _parse_attributes(d.pop("attributes", UNSET))

        def _parse_chunking_strategy(
            data: object,
        ) -> None | OtherFileChunkingStrategyObject | StaticFileChunkingStrategyObject | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                chunking_strategy_type_0 = StaticFileChunkingStrategyObject.from_dict(data)

                return chunking_strategy_type_0
            except (TypeError, ValueError, AttributeError, KeyError):
                pass
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                chunking_strategy_type_1 = OtherFileChunkingStrategyObject.from_dict(data)

                return chunking_strategy_type_1
            except (TypeError, ValueError, AttributeError, KeyError):
                pass
            return cast(None | OtherFileChunkingStrategyObject | StaticFileChunkingStrategyObject | Unset, data)

        chunking_strategy = _parse_chunking_strategy(d.pop("chunking_strategy", UNSET))

        vector_store_file = cls(
            id=id,
            created_at=created_at,
            object_=object_,
            status=status,
            usage_bytes=usage_bytes,
            vector_store_id=vector_store_id,
            last_error=last_error,
            attributes=attributes,
            chunking_strategy=chunking_strategy,
        )

        vector_store_file.additional_properties = d
        return vector_store_file

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from __future__ import annotations

import datetime
from collections.abc import Mapping
from typing import Any, TypeVar, cast

from attrs import define as _attrs_define
from attrs import field as _attrs_field
from dateutil.parser import isoparse

from ..types import UNSET, Unset

T = TypeVar("T", bound="WebhookDoc")


@_attrs_define
class WebhookDoc:
    """
    Attributes:
        user_id (str):
        runnable_id (str):
        field_id (str):
        updated_at (datetime.datetime | Unset):
        created_at (datetime.datetime | Unset):
        run_id (None | str | Unset):
        webhook_id (str | Unset):
        webhook_path (None | str | Unset):
    """

    user_id: str
    runnable_id: str
    field_id: str
    updated_at: datetime.datetime | Unset = UNSET
    created_at: datetime.datetime | Unset = UNSET
    run_id: None | str | Unset = UNSET
    webhook_id: str | Unset = UNSET
    webhook_path: None | str | Unset = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        user_id = self.user_id

        runnable_id = self.runnable_id

        field_id = self.field_id

        updated_at: str | Unset = UNSET
        if not isinstance(self.updated_at, Unset):
            updated_at = self.updated_at.isoformat()

        created_at: str | Unset = UNSET
        if not isinstance(self.created_at, Unset):
            created_at = self.created_at.isoformat()

        run_id: None | str | Unset
        if isinstance(self.run_id, Unset):
            run_id = UNSET
        else:
            run_id = self.run_id

        webhook_id = self.webhook_id

        webhook_path: None | str | Unset
        if isinstance(self.webhook_path, Unset):
            webhook_path = UNSET
        else:
            webhook_path = self.webhook_path

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "user_id": user_id,
                "runnable_id": runnable_id,
                "_id": field_id,
            }
        )
        if updated_at is not UNSET:
            field_dict["updated_at"] = updated_at
        if created_at is not UNSET:
            field_dict["created_at"] = created_at
        if run_id is not UNSET:
            field_dict["run_id"] = run_id
        if webhook_id is not UNSET:
            field_dict["webhook_id"] = webhook_id
        if webhook_path is not UNSET:
            field_dict["webhook_path"] = webhook_path

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        d = dict(src_dict)
        user_id = d.pop("user_id")

        runnable_id = d.pop("runnable_id")

        field_id = d.pop("_id")

        _updated_at = d.pop("updated_at", UNSET)
        updated_at: datetime.datetime | Unset
        if isinstance(_updated_at, Unset):
            updated_at = UNSET
        else:
            updated_at = isoparse(_updated_at)

        _created_at = d.pop("created_at", UNSET)
        created_at: datetime.datetime | Unset
        if isinstance(_created_at, Unset):
            created_at = UNSET
        else:
            created_at = isoparse(_created_at)

        def _parse_run_id(data: object) -> None | str | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(None | str | Unset, data)

        run_id = _parse_run_id(d.pop("run_id", UNSET))

        webhook_id = d.pop("webhook_id", UNSET)

        def _parse_webhook_path(data: object) -> None | str | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(None | str | Unset, data)

        webhook_path = _parse_webhook_path(d.pop("webhook_path", UNSET))

        webhook_doc = cls(
            user_id=user_id,
            runnable_id=runnable_id,
            field_id=field_id,
            updated_at=updated_at,
            created_at=created_at,
            run_id=run_id,
            webhook_id=webhook_id,
            webhook_path=webhook_path,
        )

        webhook_doc.additional_properties = d
        return webhook_doc

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from __future__ import annotations

from collections.abc import Mapping
from typing import TYPE_CHECKING, Any, TypeVar, cast

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.structured_output_schema_properties import StructuredOutputSchemaProperties


T = TypeVar("T", bound="StructuredOutputSchema")


@_attrs_define
class StructuredOutputSchema:
    """Schema for structured outputs using JSON Schema.

    Attributes:
        properties (StructuredOutputSchemaProperties):
        type_ (str | Unset):  Default: 'object'.
        required (list[str] | None | Unset):
        description (None | str | Unset):
        strict (bool | Unset):  Default: True.
    """

    properties: StructuredOutputSchemaProperties
    type_: str | Unset = "object"
    required: list[str] | None | Unset = UNSET
    description: None | str | Unset = UNSET
    strict: bool | Unset = True
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        properties = self.properties.to_dict()

        type_ = self.type_

        required: list[str] | None | Unset
        if isinstance(self.required, Unset):
            required = UNSET
        elif isinstance(self.required, list):
            required = self.required

        else:
            required = self.required

        description: None | str | Unset
        if isinstance(self.description, Unset):
            description = UNSET
        else:
            description = self.description

        strict = self.strict

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "properties": properties,
            }
        )
        if type_ is not UNSET:
            field_dict["type"] = type_
        if required is not UNSET:
            field_dict["required"] = required
        if description is not UNSET:
            field_dict["description"] = description
        if strict is not UNSET:
            field_dict["strict"] = strict

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        from ..models.structured_output_schema_properties import StructuredOutputSchemaProperties

        d = dict(src_dict)
        properties = StructuredOutputSchemaProperties.from_dict(d.pop("properties"))

        type_ = d.pop("type", UNSET)

        def _parse_required(data: object) -> list[str] | None | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, list):
                    raise TypeError()
                required_type_0 = cast(list[str], data)

                return required_type_0
            except (TypeError, ValueError, AttributeError, KeyError):
                pass
            return cast(list[str] | None | Unset, data)

        required = _parse_required(d.pop("required", UNSET))

        def _parse_description(data: object) -> None | str | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(None | str | Unset, data)

        description = _parse_description(d.pop("description", UNSET))

        strict = d.pop("strict", UNSET)

        structured_output_schema = cls(
            properties=properties,
            type_=type_,
            required=required,
            description=description,
            strict=strict,
        )

        structured_output_schema.additional_properties = d
        return structured_output_schema

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from __future__ import annotations

from collections.abc import Mapping
from typing import TYPE_CHECKING, Any, Literal, TypeVar, cast

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..models.image_edit_response_format_type_0 import ImageEditResponseFormatType0
from ..models.image_edit_size_type_0 import ImageEditSizeType0
from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.image_edit_extra_headers_type_0 import ImageEditExtraHeadersType0
    from ..models.image_edit_extra_query_type_0 import ImageEditExtraQueryType0


T = TypeVar("T", bound="ImageEdit")


@_attrs_define
class ImageEdit:
    """
    Attributes:
        image (str):
        prompt (str):
        extra_headers (ImageEditExtraHeadersType0 | None | Unset):
        extra_query (ImageEditExtraQueryType0 | None | Unset):
        extra_body (Any | None | Unset):
        timeout (float | None | Unset):
        mask (None | str | Unset):
        model (Literal['dall-e-3'] | None | str | Unset):  Default: 'dall-e-3'.
        n (int | None | Unset):  Default: 1.
        response_format (ImageEditResponseFormatType0 | None | Unset):
        size (ImageEditSizeType0 | None | Unset):
        user (None | str | Unset):
    """

    image: str
    prompt: str
    extra_headers: ImageEditExtraHeadersType0 | None | Unset = UNSET
    extra_query: ImageEditExtraQueryType0 | None | Unset = UNSET
    extra_body: Any | None | Unset = UNSET
    timeout: float | None | Unset = UNSET
    mask: None | str | Unset = UNSET
    model: Literal["dall-e-3"] | None | str | Unset = "dall-e-3"
    n: int | None | Unset = 1
    response_format: ImageEditResponseFormatType0 | None | Unset = UNSET
    size: ImageEditSizeType0 | None | Unset = UNSET
    user: None | str | Unset = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        from ..models.image_edit_extra_headers_type_0 import ImageEditExtraHeadersType0
        from ..models.image_edit_extra_query_type_0 import ImageEditExtraQueryType0

        image: str
        image = self.image

        prompt = self.prompt

        extra_headers: dict[str, Any] | None | Unset
        if isinstance(self.extra_headers, Unset):
            extra_headers = UNSET
        elif isinstance(self.extra_headers, ImageEditExtraHeadersType0):
            extra_headers = self.extra_headers.to_dict()
        else:
            extra_headers = self.extra_headers

        extra_query: dict[str, Any] | None | Unset
        if isinstance(self.extra_query, Unset):
            extra_query = UNSET
        elif isinstance(self.extra_query, ImageEditExtraQueryType0):
            extra_query = self.extra_query.to_dict()
        else:
            extra_query = self.extra_query

        extra_body: Any | None | Unset
        if isinstance(self.extra_body, Unset):
            extra_body = UNSET
        else:
            extra_body = self.extra_body

        timeout: float | None | Unset
        if isinstance(self.timeout, Unset):
            timeout = UNSET
        else:
            timeout = self.timeout

        mask: None | str | Unset
        if isinstance(self.mask, Unset):
            mask = UNSET
        else:
            mask = self.mask

        model: Literal["dall-e-3"] | None | str | Unset
        if isinstance(self.model, Unset):
            model = UNSET
        else:
            model = self.model

        n: int | None | Unset
        if isinstance(self.n, Unset):
            n = UNSET
        else:
            n = self.n

        response_format: None | str | Unset
        if isinstance(self.response_format, Unset):
            response_format = UNSET
        elif isinstance(self.response_format, ImageEditResponseFormatType0):
            response_format = self.response_format.value
        else:
            response_format = self.response_format

        size: None | str | Unset
        if isinstance(self.size, Unset):
            size = UNSET
        elif isinstance(self.size, ImageEditSizeType0):
            size = self.size.value
        else:
            size = self.size

        user: None | str | Unset
        if isinstance(self.user, Unset):
            user = UNSET
        else:
            user = self.user

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "image": image,
                "prompt": prompt,
            }
        )
        if extra_headers is not UNSET:
            field_dict["extra_headers"] = extra_headers
        if extra_query is not UNSET:
            field_dict["extra_query"] = extra_query
        if extra_body is not UNSET:
            field_dict["extra_body"] = extra_body
        if timeout is not UNSET:
            field_dict["timeout"] = timeout
        if mask is not UNSET:
            field_dict["mask"] = mask
        if model is not UNSET:
            field_dict["model"] = model
        if n is not UNSET:
            field_dict["n"] = n
        if response_format is not UNSET:
            field_dict["response_format"] = response_format
        if size is not UNSET:
            field_dict["size"] = size
        if user is not UNSET:
            field_dict["user"] = user

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        from ..models.image_edit_extra_headers_type_0 import ImageEditExtraHeadersType0
        from ..models.image_edit_extra_query_type_0 import ImageEditExtraQueryType0

        d = dict(src_dict)

        def _parse_image(data: object) -> str:
            return cast(str, data)

        image = _parse_image(d.pop("image"))

        prompt = d.pop("prompt")

        def _parse_extra_headers(data: object) -> ImageEditExtraHeadersType0 | None | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                extra_headers_type_0 = ImageEditExtraHeadersType0.from_dict(data)

                return extra_headers_type_0
            except (TypeError, ValueError, AttributeError, KeyError):
                pass
            return cast(ImageEditExtraHeadersType0 | None | Unset, data)

        extra_headers = _parse_extra_headers(d.pop("extra_headers", UNSET))

        def _parse_extra_query(data: object) -> ImageEditExtraQueryType0 | None | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                extra_query_type_0 = ImageEditExtraQueryType0.from_dict(data)

                return extra_query_type_0
            except (TypeError, ValueError, AttributeError, KeyError):
                pass
            return cast(ImageEditExtraQueryType0 | None | Unset, data)

        extra_query = _parse_extra_query(d.pop("extra_query", UNSET))

        def _parse_extra_body(data: object) -> Any | None | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Any | None | Unset, data)

        extra_body = _parse_extra_body(d.pop("extra_body", UNSET))

        def _parse_timeout(data: object) -> float | None | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(float | None | Unset, data)

        timeout = _parse_timeout(d.pop("timeout", UNSET))

        def _parse_mask(data: object) -> None | str | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(None | str | Unset, data)

        mask = _parse_mask(d.pop("mask", UNSET))

        def _parse_model(data: object) -> Literal["dall-e-3"] | None | str | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            model_type_1 = cast(Literal["dall-e-3"], data)
            if model_type_1 != "dall-e-3":
                raise ValueError(f"model_type_1 must match const 'dall-e-3', got '{model_type_1}'")
            return model_type_1
            return cast(Literal["dall-e-3"] | None | str | Unset, data)

        model = _parse_model(d.pop("model", UNSET))

        def _parse_n(data: object) -> int | None | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(int | None | Unset, data)

        n = _parse_n(d.pop("n", UNSET))

        def _parse_response_format(data: object) -> ImageEditResponseFormatType0 | None | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, str):
                    raise TypeError()
                response_format_type_0 = ImageEditResponseFormatType0(data)

                return response_format_type_0
            except (TypeError, ValueError, AttributeError, KeyError):
                pass
            return cast(ImageEditResponseFormatType0 | None | Unset, data)

        response_format = _parse_response_format(d.pop("response_format", UNSET))

        def _parse_size(data: object) -> ImageEditSizeType0 | None | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, str):
                    raise TypeError()
                size_type_0 = ImageEditSizeType0(data)

                return size_type_0
            except (TypeError, ValueError, AttributeError, KeyError):
                pass
            return cast(ImageEditSizeType0 | None | Unset, data)

        size = _parse_size(d.pop("size", UNSET))

        def _parse_user(data: object) -> None | str | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(None | str | Unset, data)

        user = _parse_user(d.pop("user", UNSET))

        image_edit = cls(
            image=image,
            prompt=prompt,
            extra_headers=extra_headers,
            extra_query=extra_query,
            extra_body=extra_body,
            timeout=timeout,
            mask=mask,
            model=model,
            n=n,
            response_format=response_format,
            size=size,
            user=user,
        )

        image_edit.additional_properties = d
        return image_edit

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from enum import Enum


class LlmChatOpenaiConfigToolChoiceType0(str, Enum):
    AUTO = "auto"
    NONE = "none"
    REQUIRED = "required"

    def __str__(self) -> str:
        return str(self.value)



from __future__ import annotations

from collections.abc import Mapping
from typing import Any, TypeVar

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

T = TypeVar("T", bound="TimeRangeConfig")


@_attrs_define
class TimeRangeConfig:
    """Configuration for time saved ranges

    Attributes:
        action_min_hours (float | Unset): Minimum hours saved per action Default: 0.5.
        action_max_hours (float | Unset): Maximum hours saved per action Default: 2.0.
        workflow_min_hours (float | Unset): Minimum hours saved per workflow Default: 2.0.
        workflow_max_hours (float | Unset): Maximum hours saved per workflow Default: 5.0.
    """

    action_min_hours: float | Unset = 0.5
    action_max_hours: float | Unset = 2.0
    workflow_min_hours: float | Unset = 2.0
    workflow_max_hours: float | Unset = 5.0
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        action_min_hours = self.action_min_hours

        action_max_hours = self.action_max_hours

        workflow_min_hours = self.workflow_min_hours

        workflow_max_hours = self.workflow_max_hours

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if action_min_hours is not UNSET:
            field_dict["action_min_hours"] = action_min_hours
        if action_max_hours is not UNSET:
            field_dict["action_max_hours"] = action_max_hours
        if workflow_min_hours is not UNSET:
            field_dict["workflow_min_hours"] = workflow_min_hours
        if workflow_max_hours is not UNSET:
            field_dict["workflow_max_hours"] = workflow_max_hours

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        d = dict(src_dict)
        action_min_hours = d.pop("action_min_hours", UNSET)

        action_max_hours = d.pop("action_max_hours", UNSET)

        workflow_min_hours = d.pop("workflow_min_hours", UNSET)

        workflow_max_hours = d.pop("workflow_max_hours", UNSET)

        time_range_config = cls(
            action_min_hours=action_min_hours,
            action_max_hours=action_max_hours,
            workflow_min_hours=workflow_min_hours,
            workflow_max_hours=workflow_max_hours,
        )

        time_range_config.additional_properties = d
        return time_range_config

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties
